// $Id: IPPort.cs,v 1.1.1.1 2007/07/03 10:15:18 tamirgal Exp $

/// <summary>************************************************************************
/// Copyright (C) 2001, Patrick Charles and Jonas Lehmann                   *
/// Distributed under the Mozilla Public License                            *
/// http://www.mozilla.org/NPL/MPL-1.1.txt                                *
/// *************************************************************************
/// </summary>
using System;
namespace Tamir.IPLib.Packets
{
	
	
	/// <summary> IP port utility class.
	/// 
	/// </summary>
	/// <author>  Patrick Charles and Jonas Lehmann
	/// </author>
	/// <version>  $Revision: 1.1.1.1 $
	/// </version>
	/// <lastModifiedBy>  $Author: tamirgal $ </lastModifiedBy>
	/// <lastModifiedAt>  $Date: 2007/07/03 10:15:18 $ </lastModifiedAt>
	public class IPPort : IPPorts
	{
		/// <summary> Fetch a well-known IP port description.</summary>
		/// <param name="port">the port associated with the description.
		/// </param>
		/// <returns> a description of the ip port.
		/// </returns>
		public static System.String getDescription(int port)
		{
			System.Int32 c = (System.Int32) port;
			if (descriptions.ContainsKey(c))
			{
				//UPGRADE_TODO: Method 'java.util.HashMap.get' was converted to 'System.Collections.Hashtable.Item' which has a different behavior. "ms-help://MS.VSCC.v80/dv_commoner/local/redirect.htm?index='!DefaultContextWindowIndex'&keyword='jlca1073_javautilHashMapget_javalangObject'"
				return (System.String) descriptions[c];
			}
			else if (port >= IPPorts_Fields.X11_BEGIN && port <= IPPorts_Fields.X11_END)
			{
				//UPGRADE_TODO: Method 'java.util.HashMap.get' was converted to 'System.Collections.Hashtable.Item' which has a different behavior. "ms-help://MS.VSCC.v80/dv_commoner/local/redirect.htm?index='!DefaultContextWindowIndex'&keyword='jlca1073_javautilHashMapget_javalangObject'"
				return (System.String) descriptions[(System.Int32) IPPorts_Fields.X11];
			}
			else
				return UNKNOWN;
		}
		
		/// <summary> Fetch a well-known IP port name.</summary>
		/// <param name="port">the port associated with the name.
		/// </param>
		/// <returns> the name of the ip port.
		/// </returns>
		public static System.String getName(int port)
		{
			System.Int32 c = (System.Int32) port;
			if (names.ContainsKey(c))
			{
				//UPGRADE_TODO: Method 'java.util.HashMap.get' was converted to 'System.Collections.Hashtable.Item' which has a different behavior. "ms-help://MS.VSCC.v80/dv_commoner/local/redirect.htm?index='!DefaultContextWindowIndex'&keyword='jlca1073_javautilHashMapget_javalangObject'"
				return (System.String) names[c];
			}
			else if (port >= IPPorts_Fields.X11_BEGIN && port <= IPPorts_Fields.X11_END)
			{
				//UPGRADE_TODO: Method 'java.util.HashMap.get' was converted to 'System.Collections.Hashtable.Item' which has a different behavior. "ms-help://MS.VSCC.v80/dv_commoner/local/redirect.htm?index='!DefaultContextWindowIndex'&keyword='jlca1073_javautilHashMapget_javalangObject'"
				return (System.String) names[(System.Int32) IPPorts_Fields.X11];
			}
			else
				return c.ToString();
		}
		
		/// <summary> 'Human-readable' IP port descriptions.</summary>
		//UPGRADE_TODO: Class 'java.util.HashMap' was converted to 'System.Collections.Hashtable' which has a different behavior. "ms-help://MS.VSCC.v80/dv_commoner/local/redirect.htm?index='!DefaultContextWindowIndex'&keyword='jlca1073_javautilHashMap'"
		private static System.Collections.Hashtable descriptions = new System.Collections.Hashtable();
		
		/// <summary> 'Human-readable' TCP port names.</summary>
		//UPGRADE_TODO: Class 'java.util.HashMap' was converted to 'System.Collections.Hashtable' which has a different behavior. "ms-help://MS.VSCC.v80/dv_commoner/local/redirect.htm?index='!DefaultContextWindowIndex'&keyword='jlca1073_javautilHashMap'"
		private static System.Collections.Hashtable names = new System.Collections.Hashtable();
		
		
		public static System.String UNKNOWN = "unknown";
		
		static IPPort()
		{
			{
				// todo: descriptions aren't descriptions
				descriptions[(System.Int32) IPPorts_Fields.NONE] = "no port specified";
				descriptions[(System.Int32) IPPorts_Fields.RESERVED_0] = "port 0 is reserved!";
				descriptions[(System.Int32) IPPorts_Fields.TCPMUX] = "tcpmux";
				descriptions[(System.Int32) IPPorts_Fields.COMPRESSNET2] = "compressnet2";
				descriptions[(System.Int32) IPPorts_Fields.COMPRESSNET] = "compressnet";
				descriptions[(System.Int32) IPPorts_Fields.RJE] = "rje";
				descriptions[(System.Int32) IPPorts_Fields.ECHO] = "echo";
				descriptions[(System.Int32) IPPorts_Fields.DISCARD] = "discard";
				descriptions[(System.Int32) IPPorts_Fields.SYSTAT] = "systat";
				descriptions[(System.Int32) IPPorts_Fields.DAYTIME] = "daytime";
				descriptions[(System.Int32) IPPorts_Fields.QOTD] = "qotd";
				descriptions[(System.Int32) IPPorts_Fields.MSP] = "msp";
				descriptions[(System.Int32) IPPorts_Fields.CHARGEN] = "chargen";
				descriptions[(System.Int32) IPPorts_Fields.FTP_DATA] = "ftp_data";
				descriptions[(System.Int32) IPPorts_Fields.FTP] = "file transfer protocol";
				descriptions[(System.Int32) IPPorts_Fields.SSH] = "secure shell protocol";
				descriptions[(System.Int32) IPPorts_Fields.TELNET] = "remote terminal protocol";
				descriptions[(System.Int32) IPPorts_Fields.PRIV_MAIL] = "priv_mail";
				descriptions[(System.Int32) IPPorts_Fields.SMTP] = "simple mail xfer protocol";
				descriptions[(System.Int32) IPPorts_Fields.NSW_FE] = "nsw_fe";
				descriptions[(System.Int32) IPPorts_Fields.MSG_ICP] = "msg_icp";
				descriptions[(System.Int32) IPPorts_Fields.MSG_AUTH] = "msg_auth";
				descriptions[(System.Int32) IPPorts_Fields.DSP] = "dsp";
				descriptions[(System.Int32) IPPorts_Fields.PRIV_PRINT] = "priv_print";
				descriptions[(System.Int32) IPPorts_Fields.TIME] = "time";
				descriptions[(System.Int32) IPPorts_Fields.RAP] = "rap";
				descriptions[(System.Int32) IPPorts_Fields.RLP] = "rlp";
				descriptions[(System.Int32) IPPorts_Fields.GRAPHICS] = "graphics";
				descriptions[(System.Int32) IPPorts_Fields.NAME] = "name";
				descriptions[(System.Int32) IPPorts_Fields.NAMESERVER] = "nameserver";
				descriptions[(System.Int32) IPPorts_Fields.NICNAME] = "nicname";
				descriptions[(System.Int32) IPPorts_Fields.MPM_FLAGS] = "mpm_flags";
				descriptions[(System.Int32) IPPorts_Fields.MPM] = "mpm";
				descriptions[(System.Int32) IPPorts_Fields.MPM_SND] = "mpm_snd";
				descriptions[(System.Int32) IPPorts_Fields.NI_FTP] = "ni_ftp";
				descriptions[(System.Int32) IPPorts_Fields.AUDITD] = "auditd";
				descriptions[(System.Int32) IPPorts_Fields.TACACS] = "tacacs";
				descriptions[(System.Int32) IPPorts_Fields.RE_MAIL_CK] = "re_mail_ck";
				descriptions[(System.Int32) IPPorts_Fields.LA_MAINT] = "la_maint";
				descriptions[(System.Int32) IPPorts_Fields.XNS_TIME] = "xns_time";
				descriptions[(System.Int32) IPPorts_Fields.DOMAIN] = "domain";
				descriptions[(System.Int32) IPPorts_Fields.XNS_CH] = "xns_ch";
				descriptions[(System.Int32) IPPorts_Fields.ISI_GL] = "isi_gl";
				descriptions[(System.Int32) IPPorts_Fields.XNS_AUTH] = "xns_auth";
				descriptions[(System.Int32) IPPorts_Fields.PRIV_TERMINAL] = "priv_terminal";
				descriptions[(System.Int32) IPPorts_Fields.XNS_MAIL] = "xns_mail";
				descriptions[(System.Int32) IPPorts_Fields.PRIV_FILE] = "priv_file";
				descriptions[(System.Int32) IPPorts_Fields.NI_MAIL] = "ni_mail";
				descriptions[(System.Int32) IPPorts_Fields.ACAS] = "acas";
				descriptions[(System.Int32) IPPorts_Fields.WHOIS] = "whois++";
				descriptions[(System.Int32) IPPorts_Fields.COVIA] = "covia";
				descriptions[(System.Int32) IPPorts_Fields.TACACS_DS] = "tacacs_ds";
				descriptions[(System.Int32) IPPorts_Fields.SQLSNET] = "sql*net";
				descriptions[(System.Int32) IPPorts_Fields.BOOTPS] = "bootps";
				descriptions[(System.Int32) IPPorts_Fields.BOOTPC] = "bootpc";
				descriptions[(System.Int32) IPPorts_Fields.TFTP] = "tftp";
				descriptions[(System.Int32) IPPorts_Fields.GOPHER] = "gopher";
				descriptions[(System.Int32) IPPorts_Fields.NETRJS_1] = "netrjs_1";
				descriptions[(System.Int32) IPPorts_Fields.NETRJS_2] = "netrjs_2";
				descriptions[(System.Int32) IPPorts_Fields.NETRJS_3] = "netrjs_3";
				descriptions[(System.Int32) IPPorts_Fields.NETRJS_4] = "netrjs_4";
				descriptions[(System.Int32) IPPorts_Fields.PRIV_DIALOUT] = "priv_dialout";
				descriptions[(System.Int32) IPPorts_Fields.DEOS] = "deos";
				descriptions[(System.Int32) IPPorts_Fields.PRIV_RJE] = "priv_rje";
				descriptions[(System.Int32) IPPorts_Fields.VETTCP] = "vettcp";
				descriptions[(System.Int32) IPPorts_Fields.FINGER] = "finger";
				descriptions[(System.Int32) IPPorts_Fields.HTTP] = "http";
				descriptions[(System.Int32) IPPorts_Fields.WWW] = "www";
				descriptions[(System.Int32) IPPorts_Fields.HOSTS2_NS] = "hosts2_ns";
				descriptions[(System.Int32) IPPorts_Fields.XFER] = "xfer";
				descriptions[(System.Int32) IPPorts_Fields.MIT_ML_DEV] = "mit_ml_dev";
				descriptions[(System.Int32) IPPorts_Fields.CTF] = "ctf";
				descriptions[(System.Int32) IPPorts_Fields.MIT_ML_DEV2] = "mit_ml_dev";
				descriptions[(System.Int32) IPPorts_Fields.MFCOBOL] = "mfcobol";
				descriptions[(System.Int32) IPPorts_Fields.PRIV_TERMLINK] = "priv_termlink";
				descriptions[(System.Int32) IPPorts_Fields.KERBEROS] = "kerberos";
				descriptions[(System.Int32) IPPorts_Fields.SU_MIT_TG] = "su_mit_tg";
				descriptions[(System.Int32) IPPorts_Fields.DNSIX] = "dnsix";
				descriptions[(System.Int32) IPPorts_Fields.MIT_DOV] = "mit_dov";
				descriptions[(System.Int32) IPPorts_Fields.NPP] = "npp";
				descriptions[(System.Int32) IPPorts_Fields.DCP] = "dcp";
				descriptions[(System.Int32) IPPorts_Fields.OBJCALL] = "objcall";
				descriptions[(System.Int32) IPPorts_Fields.SUPDUP] = "supdup";
				descriptions[(System.Int32) IPPorts_Fields.DIXIE] = "dixie";
				descriptions[(System.Int32) IPPorts_Fields.SWIFT_RVF] = "swift_rvf";
				descriptions[(System.Int32) IPPorts_Fields.TACNEWS] = "tacnews";
				descriptions[(System.Int32) IPPorts_Fields.METAGRAM] = "metagram";
				descriptions[(System.Int32) IPPorts_Fields.NEWACCT] = "newacct";
				descriptions[(System.Int32) IPPorts_Fields.HOSTNAME] = "hostname";
				descriptions[(System.Int32) IPPorts_Fields.ISO_TSAP] = "iso_tsap";
				descriptions[(System.Int32) IPPorts_Fields.GPPITNP] = "gppitnp";
				descriptions[(System.Int32) IPPorts_Fields.ACR_NEMA] = "acr_nema";
				descriptions[(System.Int32) IPPorts_Fields.CSO] = "cso";
				descriptions[(System.Int32) IPPorts_Fields.CSNET_NS] = "csnet_ns";
				descriptions[(System.Int32) IPPorts_Fields.TSMUX_3COM] = "3com_tsmux";
				descriptions[(System.Int32) IPPorts_Fields.RTELNET] = "rtelnet";
				descriptions[(System.Int32) IPPorts_Fields.SNAGAS] = "snagas";
				descriptions[(System.Int32) IPPorts_Fields.POP2] = "pop2";
				descriptions[(System.Int32) IPPorts_Fields.POP3] = "pop3";
				descriptions[(System.Int32) IPPorts_Fields.SUNRPC] = "sunrpc";
				descriptions[(System.Int32) IPPorts_Fields.MCIDAS] = "mcidas";
				descriptions[(System.Int32) IPPorts_Fields.IDENT] = "ident";
				descriptions[(System.Int32) IPPorts_Fields.AUTH] = "auth";
				descriptions[(System.Int32) IPPorts_Fields.AUDIONEWS] = "audionews";
				descriptions[(System.Int32) IPPorts_Fields.SFTP] = "sftp";
				descriptions[(System.Int32) IPPorts_Fields.ANSANOTIFY] = "ansanotify";
				descriptions[(System.Int32) IPPorts_Fields.UUCP_PATH] = "uucp_path";
				descriptions[(System.Int32) IPPorts_Fields.SQLSERV] = "sqlserv";
				descriptions[(System.Int32) IPPorts_Fields.NNTP] = "nntp";
				descriptions[(System.Int32) IPPorts_Fields.CFDPTKT] = "cfdptkt";
				descriptions[(System.Int32) IPPorts_Fields.ERPC] = "erpc";
				descriptions[(System.Int32) IPPorts_Fields.SMAKYNET] = "smakynet";
				descriptions[(System.Int32) IPPorts_Fields.NTP] = "ntp";
				descriptions[(System.Int32) IPPorts_Fields.ANSATRADER] = "ansatrader";
				descriptions[(System.Int32) IPPorts_Fields.LOCUS_MAP] = "locus_map";
				descriptions[(System.Int32) IPPorts_Fields.NXEDIT] = "nxedit";
				descriptions[(System.Int32) IPPorts_Fields.LOCUS_CON] = "locus_con";
				descriptions[(System.Int32) IPPorts_Fields.GSS_XLICEN] = "gss_xlicen";
				descriptions[(System.Int32) IPPorts_Fields.PWDGEN] = "pwdgen";
				descriptions[(System.Int32) IPPorts_Fields.CISCO_FNA] = "cisco_fna";
				descriptions[(System.Int32) IPPorts_Fields.CISCO_TNA] = "cisco_tna";
				descriptions[(System.Int32) IPPorts_Fields.CISCO_SYS] = "cisco_sys";
				descriptions[(System.Int32) IPPorts_Fields.STATSRV] = "statsrv";
				descriptions[(System.Int32) IPPorts_Fields.INGRES_NET] = "ingres_net";
				descriptions[(System.Int32) IPPorts_Fields.EPMAP] = "epmap";
				descriptions[(System.Int32) IPPorts_Fields.PROFILE] = "profile";
				descriptions[(System.Int32) IPPorts_Fields.NETBIOS_NS] = "netbios_ns";
				descriptions[(System.Int32) IPPorts_Fields.NETBIOS_DGM] = "netbios_dgm";
				descriptions[(System.Int32) IPPorts_Fields.NETBIOS_SSN] = "netbios_ssn";
				descriptions[(System.Int32) IPPorts_Fields.EMFIS_DATA] = "emfis_data";
				descriptions[(System.Int32) IPPorts_Fields.EMFIS_CNTL] = "emfis_cntl";
				descriptions[(System.Int32) IPPorts_Fields.BL_IDM] = "bl_idm";
				descriptions[(System.Int32) IPPorts_Fields.IMAP] = "imap";
				descriptions[(System.Int32) IPPorts_Fields.UMA] = "uma";
				descriptions[(System.Int32) IPPorts_Fields.UAAC] = "uaac";
				descriptions[(System.Int32) IPPorts_Fields.ISO_TP0] = "iso_tp0";
				descriptions[(System.Int32) IPPorts_Fields.ISO_IP] = "iso_ip";
				descriptions[(System.Int32) IPPorts_Fields.JARGON] = "jargon";
				descriptions[(System.Int32) IPPorts_Fields.AED_512] = "aed_512";
				descriptions[(System.Int32) IPPorts_Fields.SQL_NET] = "sql_net";
				descriptions[(System.Int32) IPPorts_Fields.HEMS] = "hems";
				descriptions[(System.Int32) IPPorts_Fields.BFTP] = "bftp";
				descriptions[(System.Int32) IPPorts_Fields.SGMP] = "sgmp";
				descriptions[(System.Int32) IPPorts_Fields.NETSC_PROD] = "netsc_prod";
				descriptions[(System.Int32) IPPorts_Fields.NETSC_DEV] = "netsc_dev";
				descriptions[(System.Int32) IPPorts_Fields.SQLSRV] = "sqlsrv";
				descriptions[(System.Int32) IPPorts_Fields.KNET_CMP] = "knet_cmp";
				descriptions[(System.Int32) IPPorts_Fields.PCMAIL_SRV] = "pcmail_srv";
				descriptions[(System.Int32) IPPorts_Fields.NSS_ROUTING] = "nss_routing";
				descriptions[(System.Int32) IPPorts_Fields.SGMP_TRAPS] = "sgmp_traps";
				descriptions[(System.Int32) IPPorts_Fields.SNMP] = "snmp";
				descriptions[(System.Int32) IPPorts_Fields.SNMPTRAP] = "snmptrap";
				descriptions[(System.Int32) IPPorts_Fields.CMIP_MAN] = "cmip_man";
				descriptions[(System.Int32) IPPorts_Fields.CMIP_AGENT] = "cmip_agent";
				descriptions[(System.Int32) IPPorts_Fields.XNS_COURIER] = "xns_courier";
				descriptions[(System.Int32) IPPorts_Fields.S_NET] = "s_net";
				descriptions[(System.Int32) IPPorts_Fields.NAMP] = "namp";
				descriptions[(System.Int32) IPPorts_Fields.RSVD] = "rsvd";
				descriptions[(System.Int32) IPPorts_Fields.SEND] = "send";
				descriptions[(System.Int32) IPPorts_Fields.PRINT_SRV] = "print_srv";
				descriptions[(System.Int32) IPPorts_Fields.MULTIPLEX] = "multiplex";
				descriptions[(System.Int32) IPPorts_Fields.CL1] = "cl1";
				descriptions[(System.Int32) IPPorts_Fields.XYPLEX_MUX] = "xyplex_mux";
				descriptions[(System.Int32) IPPorts_Fields.MAILQ] = "mailq";
				descriptions[(System.Int32) IPPorts_Fields.VMNET] = "vmnet";
				descriptions[(System.Int32) IPPorts_Fields.GENRAD_MUX] = "genrad_mux";
				descriptions[(System.Int32) IPPorts_Fields.XDMCP] = "xdmcp";
				descriptions[(System.Int32) IPPorts_Fields.NEXTSTEP] = "nextstep";
				descriptions[(System.Int32) IPPorts_Fields.BGP] = "bgp";
				descriptions[(System.Int32) IPPorts_Fields.RIS] = "ris";
				descriptions[(System.Int32) IPPorts_Fields.UNIFY] = "unify";
				descriptions[(System.Int32) IPPorts_Fields.AUDIT] = "audit";
				descriptions[(System.Int32) IPPorts_Fields.OCBINDER] = "ocbinder";
				descriptions[(System.Int32) IPPorts_Fields.OCSERVER] = "ocserver";
				descriptions[(System.Int32) IPPorts_Fields.REMOTE_KIS] = "remote_kis";
				descriptions[(System.Int32) IPPorts_Fields.KIS] = "kis";
				descriptions[(System.Int32) IPPorts_Fields.ACI] = "aci";
				descriptions[(System.Int32) IPPorts_Fields.MUMPS] = "mumps";
				descriptions[(System.Int32) IPPorts_Fields.QFT] = "qft";
				descriptions[(System.Int32) IPPorts_Fields.GACP] = "gacp";
				descriptions[(System.Int32) IPPorts_Fields.PROSPERO] = "prospero";
				descriptions[(System.Int32) IPPorts_Fields.OSU_NMS] = "osu_nms";
				descriptions[(System.Int32) IPPorts_Fields.SRMP] = "srmp";
				descriptions[(System.Int32) IPPorts_Fields.IRC] = "irc";
				descriptions[(System.Int32) IPPorts_Fields.DN6_NLM_AUD] = "dn6_nlm_aud";
				descriptions[(System.Int32) IPPorts_Fields.DN6_SMM_RED] = "dn6_smm_red";
				descriptions[(System.Int32) IPPorts_Fields.DLS] = "dls";
				descriptions[(System.Int32) IPPorts_Fields.DLS_MON] = "dls_mon";
				descriptions[(System.Int32) IPPorts_Fields.SMUX] = "smux";
				descriptions[(System.Int32) IPPorts_Fields.SRC] = "src";
				descriptions[(System.Int32) IPPorts_Fields.AT_RTMP] = "at_rtmp";
				descriptions[(System.Int32) IPPorts_Fields.AT_NBP] = "at_nbp";
				descriptions[(System.Int32) IPPorts_Fields.AT_3] = "at_3";
				descriptions[(System.Int32) IPPorts_Fields.AT_ECHO] = "at_echo";
				descriptions[(System.Int32) IPPorts_Fields.AT_5] = "at_5";
				descriptions[(System.Int32) IPPorts_Fields.AT_ZIS] = "at_zis";
				descriptions[(System.Int32) IPPorts_Fields.AT_7] = "at_7";
				descriptions[(System.Int32) IPPorts_Fields.AT_8] = "at_8";
				descriptions[(System.Int32) IPPorts_Fields.QMTP] = "qmtp";
				descriptions[(System.Int32) IPPorts_Fields.Z39_50] = "z39.50";
				descriptions[(System.Int32) IPPorts_Fields.N914C] = "914c_g";
				descriptions[(System.Int32) IPPorts_Fields.ANET] = "anet";
				descriptions[(System.Int32) IPPorts_Fields.IPX] = "ipx";
				descriptions[(System.Int32) IPPorts_Fields.VMPWSCS] = "vmpwscs";
				descriptions[(System.Int32) IPPorts_Fields.SOFTPC] = "softpc";
				descriptions[(System.Int32) IPPorts_Fields.CAILIC] = "CAIlic";
				descriptions[(System.Int32) IPPorts_Fields.DBASE] = "dbase";
				descriptions[(System.Int32) IPPorts_Fields.MPP] = "mpp";
				descriptions[(System.Int32) IPPorts_Fields.UARPS] = "uarps";
				descriptions[(System.Int32) IPPorts_Fields.IMAP3] = "imap3";
				descriptions[(System.Int32) IPPorts_Fields.FLN_SPX] = "fln_spx";
				descriptions[(System.Int32) IPPorts_Fields.RSH_SPX] = "rsh_spx";
				descriptions[(System.Int32) IPPorts_Fields.CDC] = "cdc";
				descriptions[(System.Int32) IPPorts_Fields.MASQDIALER] = "masqdialer";
				descriptions[(System.Int32) IPPorts_Fields.DIRECT] = "direct";
				descriptions[(System.Int32) IPPorts_Fields.SUR_MEAS] = "sur_meas";
				descriptions[(System.Int32) IPPorts_Fields.INBUSINESS] = "inbusiness";
				descriptions[(System.Int32) IPPorts_Fields.LINK] = "link";
				descriptions[(System.Int32) IPPorts_Fields.DSP3270] = "dsp3270";
				descriptions[(System.Int32) IPPorts_Fields.SUBNTBCST_TFTP] = "subntbcst_tftp";
				descriptions[(System.Int32) IPPorts_Fields.BHFHS] = "bhfhs";
				descriptions[(System.Int32) IPPorts_Fields.RAP2] = "rap2";
				descriptions[(System.Int32) IPPorts_Fields.SET] = "set";
				descriptions[(System.Int32) IPPorts_Fields.YAK_CHAT] = "yak_chat";
				descriptions[(System.Int32) IPPorts_Fields.ESRO_GEN] = "esro_gen";
				descriptions[(System.Int32) IPPorts_Fields.OPENPORT] = "openport";
				descriptions[(System.Int32) IPPorts_Fields.NSIIOPS] = "nsiiops";
				descriptions[(System.Int32) IPPorts_Fields.ARCISDMS] = "arcisdms";
				descriptions[(System.Int32) IPPorts_Fields.HDAP] = "hdap";
				descriptions[(System.Int32) IPPorts_Fields.BGMP] = "bgmp";
				descriptions[(System.Int32) IPPorts_Fields.X_BONE_CTL] = "x_bone_ctl";
				descriptions[(System.Int32) IPPorts_Fields.SST] = "sst";
				descriptions[(System.Int32) IPPorts_Fields.TD_SERVICE] = "td_service";
				descriptions[(System.Int32) IPPorts_Fields.TD_REPLICA] = "td_replica";
				descriptions[(System.Int32) IPPorts_Fields.HTTP_MGMT] = "http_mgmt";
				descriptions[(System.Int32) IPPorts_Fields.PERSONAL_LINK] = "personal_link";
				descriptions[(System.Int32) IPPorts_Fields.CABLEPORT_AX] = "cableport_ax";
				descriptions[(System.Int32) IPPorts_Fields.RESCAP] = "rescap";
				descriptions[(System.Int32) IPPorts_Fields.CORERJD] = "corerjd";
				descriptions[(System.Int32) IPPorts_Fields.FXP_1] = "fxp_1";
				descriptions[(System.Int32) IPPorts_Fields.K_BLOCK] = "k_block";
				descriptions[(System.Int32) IPPorts_Fields.NOVASTORBAKCUP] = "novastorbakcup";
				descriptions[(System.Int32) IPPorts_Fields.ENTRUSTTIME] = "entrusttime";
				descriptions[(System.Int32) IPPorts_Fields.BHMDS] = "bhmds";
				descriptions[(System.Int32) IPPorts_Fields.ASIP_WEBADMIN] = "asip_webadmin";
				descriptions[(System.Int32) IPPorts_Fields.VSLMP] = "vslmp";
				descriptions[(System.Int32) IPPorts_Fields.MAGENTA_LOGIC] = "magenta_logic";
				descriptions[(System.Int32) IPPorts_Fields.OPALIS_ROBOT] = "opalis_robot";
				descriptions[(System.Int32) IPPorts_Fields.DPSI] = "dpsi";
				descriptions[(System.Int32) IPPorts_Fields.DECAUTH] = "decauth";
				descriptions[(System.Int32) IPPorts_Fields.ZANNET] = "zannet";
				descriptions[(System.Int32) IPPorts_Fields.PKIX_TIMESTAMP] = "pkix_timestamp";
				descriptions[(System.Int32) IPPorts_Fields.PTP_EVENT] = "ptp_event";
				descriptions[(System.Int32) IPPorts_Fields.PTP_GENERAL] = "ptp_general";
				descriptions[(System.Int32) IPPorts_Fields.PIP] = "pip";
				descriptions[(System.Int32) IPPorts_Fields.RTSPS] = "rtsps";
				descriptions[(System.Int32) IPPorts_Fields.TEXAR] = "texar";
				descriptions[(System.Int32) IPPorts_Fields.PDAP] = "pdap";
				descriptions[(System.Int32) IPPorts_Fields.PAWSERV] = "pawserv";
				descriptions[(System.Int32) IPPorts_Fields.ZSERV] = "zserv";
				descriptions[(System.Int32) IPPorts_Fields.FATSERV] = "fatserv";
				descriptions[(System.Int32) IPPorts_Fields.CSI_SGWP] = "csi_sgwp";
				descriptions[(System.Int32) IPPorts_Fields.MFTP] = "mftp";
				descriptions[(System.Int32) IPPorts_Fields.MATIP_TYPE_A] = "matip_type_a";
				descriptions[(System.Int32) IPPorts_Fields.MATIP_TYPE_B] = "matip_type_b";
				descriptions[(System.Int32) IPPorts_Fields.BHOETTY] = "bhoetty";
				descriptions[(System.Int32) IPPorts_Fields.DTAG_STE_SB] = "dtag_ste_sb";
				descriptions[(System.Int32) IPPorts_Fields.BHOEDAP4] = "bhoedap4";
				descriptions[(System.Int32) IPPorts_Fields.NDSAUTH] = "ndsauth";
				descriptions[(System.Int32) IPPorts_Fields.BH611] = "bh611";
				descriptions[(System.Int32) IPPorts_Fields.DATEX_ASN] = "datex_asn";
				descriptions[(System.Int32) IPPorts_Fields.CLOANTO_NET] = "cloanto_net";
				descriptions[(System.Int32) IPPorts_Fields.BHEVENT] = "bhevent";
				descriptions[(System.Int32) IPPorts_Fields.SHRINKWRAP] = "shrinkwrap";
				descriptions[(System.Int32) IPPorts_Fields.NSRMP] = "nsrmp";
				descriptions[(System.Int32) IPPorts_Fields.SCOI2ODIALOG] = "scoi2odialog";
				descriptions[(System.Int32) IPPorts_Fields.SEMANTIX] = "semantix";
				descriptions[(System.Int32) IPPorts_Fields.SRSSEND] = "srssend";
				descriptions[(System.Int32) IPPorts_Fields.RSVP_TUNNEL] = "rsvp_tunnel";
				descriptions[(System.Int32) IPPorts_Fields.AURORA_CMGR] = "aurora_cmgr";
				descriptions[(System.Int32) IPPorts_Fields.DTK] = "dtk";
				descriptions[(System.Int32) IPPorts_Fields.ODMR] = "odmr";
				descriptions[(System.Int32) IPPorts_Fields.MORTGAGEWARE] = "mortgageware";
				descriptions[(System.Int32) IPPorts_Fields.QBIKGDP] = "qbikgdp";
				descriptions[(System.Int32) IPPorts_Fields.RPC2PORTMAP] = "rpc2portmap";
				descriptions[(System.Int32) IPPorts_Fields.CODAAUTH] = "codaauth";
				descriptions[(System.Int32) IPPorts_Fields.CLEARCASE] = "clearcase";
				descriptions[(System.Int32) IPPorts_Fields.ULISTPROC] = "ulistproc";
				descriptions[(System.Int32) IPPorts_Fields.LEGENT_1] = "legent_1";
				descriptions[(System.Int32) IPPorts_Fields.LEGENT_2] = "legent_2";
				descriptions[(System.Int32) IPPorts_Fields.HASSLE] = "hassle";
				descriptions[(System.Int32) IPPorts_Fields.NIP] = "nip";
				descriptions[(System.Int32) IPPorts_Fields.TNETOS] = "tnETOS";
				descriptions[(System.Int32) IPPorts_Fields.DSETOS] = "dsETOS";
				descriptions[(System.Int32) IPPorts_Fields.IS99C] = "is99c";
				descriptions[(System.Int32) IPPorts_Fields.IS99S] = "is99s";
				descriptions[(System.Int32) IPPorts_Fields.HP_COLLECTOR] = "hp_collector";
				descriptions[(System.Int32) IPPorts_Fields.HP_MANAGED_NODE] = "hp_managed_node";
				descriptions[(System.Int32) IPPorts_Fields.HP_ALARM_MGR] = "hp_alarm_mgr";
				descriptions[(System.Int32) IPPorts_Fields.ARNS] = "arns";
				descriptions[(System.Int32) IPPorts_Fields.IBM_APP] = "ibm_app";
				descriptions[(System.Int32) IPPorts_Fields.ASA] = "asa";
				descriptions[(System.Int32) IPPorts_Fields.AURP] = "aurp";
				descriptions[(System.Int32) IPPorts_Fields.UNIDATA_LDM] = "unidata_ldm";
				descriptions[(System.Int32) IPPorts_Fields.UIS] = "uis";
				descriptions[(System.Int32) IPPorts_Fields.SYNOTICS_RELAY] = "synotics_relay";
				descriptions[(System.Int32) IPPorts_Fields.SYNOTICS_BROKER] = "synotics_broker";
				descriptions[(System.Int32) IPPorts_Fields.META5] = "meta5";
				descriptions[(System.Int32) IPPorts_Fields.EMBL_NDT] = "embl_ndt";
				descriptions[(System.Int32) IPPorts_Fields.NETCP] = "netcp";
				descriptions[(System.Int32) IPPorts_Fields.NETWARE_IP] = "netware_ip";
				descriptions[(System.Int32) IPPorts_Fields.MPTN] = "mptn";
				descriptions[(System.Int32) IPPorts_Fields.KRYPTOLAN] = "kryptolan";
				descriptions[(System.Int32) IPPorts_Fields.ISO_TSAP_C2] = "iso_tsap_c2";
				descriptions[(System.Int32) IPPorts_Fields.WORK_SOL] = "work_sol";
				descriptions[(System.Int32) IPPorts_Fields.UPS] = "ups";
				descriptions[(System.Int32) IPPorts_Fields.GENIE] = "genie";
				descriptions[(System.Int32) IPPorts_Fields.DECAP] = "decap";
				descriptions[(System.Int32) IPPorts_Fields.NCED] = "nced";
				descriptions[(System.Int32) IPPorts_Fields.NCLD] = "ncld";
				descriptions[(System.Int32) IPPorts_Fields.IMSP] = "imsp";
				descriptions[(System.Int32) IPPorts_Fields.TIMBUKTU] = "timbuktu";
				descriptions[(System.Int32) IPPorts_Fields.PRM_SM] = "prm_sm";
				descriptions[(System.Int32) IPPorts_Fields.PRM_NM] = "prm_nm";
				descriptions[(System.Int32) IPPorts_Fields.DECLADEBUG] = "decladebug";
				descriptions[(System.Int32) IPPorts_Fields.RMT] = "rmt";
				descriptions[(System.Int32) IPPorts_Fields.SYNOPTICS_TRAP] = "synoptics_trap";
				descriptions[(System.Int32) IPPorts_Fields.SMSP] = "smsp";
				descriptions[(System.Int32) IPPorts_Fields.INFOSEEK] = "infoseek";
				descriptions[(System.Int32) IPPorts_Fields.BNET] = "bnet";
				descriptions[(System.Int32) IPPorts_Fields.SILVERPLATTER] = "silverplatter";
				descriptions[(System.Int32) IPPorts_Fields.ONMUX] = "onmux";
				descriptions[(System.Int32) IPPorts_Fields.HYPER_G] = "hyper_g";
				descriptions[(System.Int32) IPPorts_Fields.ARIEL1] = "ariel1";
				descriptions[(System.Int32) IPPorts_Fields.SMPTE] = "smpte";
				descriptions[(System.Int32) IPPorts_Fields.ARIEL2] = "ariel2";
				descriptions[(System.Int32) IPPorts_Fields.ARIEL3] = "ariel3";
				descriptions[(System.Int32) IPPorts_Fields.OPC_JOB_START] = "opc_job_start";
				descriptions[(System.Int32) IPPorts_Fields.OPC_JOB_TRACK] = "opc_job_track";
				descriptions[(System.Int32) IPPorts_Fields.ICAD_EL] = "icad_el";
				descriptions[(System.Int32) IPPorts_Fields.SMARTSDP] = "smartsdp";
				descriptions[(System.Int32) IPPorts_Fields.SVRLOC] = "svrloc";
				descriptions[(System.Int32) IPPorts_Fields.OCS_CMU] = "ocs_cmu";
				descriptions[(System.Int32) IPPorts_Fields.OCS_AMU] = "ocs_amu";
				descriptions[(System.Int32) IPPorts_Fields.UTMPSD] = "utmpsd";
				descriptions[(System.Int32) IPPorts_Fields.UTMPCD] = "utmpcd";
				descriptions[(System.Int32) IPPorts_Fields.IASD] = "iasd";
				descriptions[(System.Int32) IPPorts_Fields.NNSP] = "nnsp";
				descriptions[(System.Int32) IPPorts_Fields.MOBILEIP_AGENT] = "mobileip_agent";
				descriptions[(System.Int32) IPPorts_Fields.MOBILIP_MN] = "mobilip_mn";
				descriptions[(System.Int32) IPPorts_Fields.DNA_CML] = "dna_cml";
				descriptions[(System.Int32) IPPorts_Fields.COMSCM] = "comscm";
				descriptions[(System.Int32) IPPorts_Fields.DSFGW] = "dsfgw";
				descriptions[(System.Int32) IPPorts_Fields.DASP] = "dasp";
				descriptions[(System.Int32) IPPorts_Fields.SGCP] = "sgcp";
				descriptions[(System.Int32) IPPorts_Fields.DECVMS_SYSMGT] = "decvms_sysmgt";
				descriptions[(System.Int32) IPPorts_Fields.CVC_HOSTD] = "cvc_hostd";
				descriptions[(System.Int32) IPPorts_Fields.HTTPS] = "https";
				descriptions[(System.Int32) IPPorts_Fields.SNPP] = "snpp";
				descriptions[(System.Int32) IPPorts_Fields.MICROSOFT_DS] = "microsoft_ds";
				descriptions[(System.Int32) IPPorts_Fields.DDM_RDB] = "ddm_rdb";
				descriptions[(System.Int32) IPPorts_Fields.DDM_DFM] = "ddm_dfm";
				descriptions[(System.Int32) IPPorts_Fields.DDM_SSL] = "ddm_ssl";
				descriptions[(System.Int32) IPPorts_Fields.AS_SERVERMAP] = "as_servermap";
				descriptions[(System.Int32) IPPorts_Fields.TSERVER] = "tserver";
				descriptions[(System.Int32) IPPorts_Fields.SFS_SMP_NET] = "sfs_smp_net";
				descriptions[(System.Int32) IPPorts_Fields.SFS_CONFIG] = "sfs_config";
				descriptions[(System.Int32) IPPorts_Fields.CREATIVESERVER] = "creativeserver";
				descriptions[(System.Int32) IPPorts_Fields.CONTENTSERVER] = "contentserver";
				descriptions[(System.Int32) IPPorts_Fields.CREATIVEPARTNR] = "creativepartnr";
				descriptions[(System.Int32) IPPorts_Fields.MACON_TCP] = "macon_tcp";
				descriptions[(System.Int32) IPPorts_Fields.SCOHELP] = "scohelp";
				descriptions[(System.Int32) IPPorts_Fields.APPLEQTC] = "appleqtc";
				descriptions[(System.Int32) IPPorts_Fields.AMPR_RCMD] = "ampr_rcmd";
				descriptions[(System.Int32) IPPorts_Fields.SKRONK] = "skronk";
				descriptions[(System.Int32) IPPorts_Fields.DATASURFSRV] = "datasurfsrv";
				descriptions[(System.Int32) IPPorts_Fields.DATASURFSRVSEC] = "datasurfsrvsec";
				descriptions[(System.Int32) IPPorts_Fields.ALPES] = "alpes";
				descriptions[(System.Int32) IPPorts_Fields.KPASSWD] = "kpasswd";
				descriptions[(System.Int32) IPPorts_Fields.DIGITAL_VRC] = "digital_vrc";
				descriptions[(System.Int32) IPPorts_Fields.MYLEX_MAPD] = "mylex_mapd";
				descriptions[(System.Int32) IPPorts_Fields.PHOTURIS] = "photuris";
				descriptions[(System.Int32) IPPorts_Fields.RCP] = "rcp";
				descriptions[(System.Int32) IPPorts_Fields.SCX_PROXY] = "scx_proxy";
				descriptions[(System.Int32) IPPorts_Fields.MONDEX] = "mondex";
				descriptions[(System.Int32) IPPorts_Fields.LJK_LOGIN] = "ljk_login";
				descriptions[(System.Int32) IPPorts_Fields.HYBRID_POP] = "hybrid_pop";
				descriptions[(System.Int32) IPPorts_Fields.TN_TL_W1] = "tn_tl_w1";
				descriptions[(System.Int32) IPPorts_Fields.TCPNETHASPSRV] = "tcpnethaspsrv";
				descriptions[(System.Int32) IPPorts_Fields.TN_TL_FD1] = "tn_tl_fd1";
				descriptions[(System.Int32) IPPorts_Fields.SS7NS] = "ss7ns";
				descriptions[(System.Int32) IPPorts_Fields.SPSC] = "spsc";
				descriptions[(System.Int32) IPPorts_Fields.IAFSERVER] = "iafserver";
				descriptions[(System.Int32) IPPorts_Fields.IAFDBASE] = "iafdbase";
				descriptions[(System.Int32) IPPorts_Fields.PH] = "ph";
				descriptions[(System.Int32) IPPorts_Fields.BGS_NSI] = "bgs_nsi";
				descriptions[(System.Int32) IPPorts_Fields.ULPNET] = "ulpnet";
				descriptions[(System.Int32) IPPorts_Fields.INTEGRA_SME] = "integra_sme";
				descriptions[(System.Int32) IPPorts_Fields.POWERBURST] = "powerburst";
				descriptions[(System.Int32) IPPorts_Fields.AVIAN] = "avian";
				descriptions[(System.Int32) IPPorts_Fields.SAFT] = "saft";
				descriptions[(System.Int32) IPPorts_Fields.GSS_HTTP] = "gss_http";
				descriptions[(System.Int32) IPPorts_Fields.NEST_PROTOCOL] = "nest_protocol";
				descriptions[(System.Int32) IPPorts_Fields.MICOM_PFS] = "micom_pfs";
				descriptions[(System.Int32) IPPorts_Fields.GO_LOGIN] = "go_login";
				descriptions[(System.Int32) IPPorts_Fields.TICF_1] = "ticf_1";
				descriptions[(System.Int32) IPPorts_Fields.TICF_2] = "ticf_2";
				descriptions[(System.Int32) IPPorts_Fields.POV_RAY] = "pov_ray";
				descriptions[(System.Int32) IPPorts_Fields.INTECOURIER] = "intecourier";
				descriptions[(System.Int32) IPPorts_Fields.PIM_RP_DISC] = "pim_rp_disc";
				descriptions[(System.Int32) IPPorts_Fields.DANTZ] = "dantz";
				descriptions[(System.Int32) IPPorts_Fields.SIAM] = "siam";
				descriptions[(System.Int32) IPPorts_Fields.ISO_ILL] = "iso_ill";
				descriptions[(System.Int32) IPPorts_Fields.ISAKMP] = "isakmp";
				descriptions[(System.Int32) IPPorts_Fields.STMF] = "stmf";
				descriptions[(System.Int32) IPPorts_Fields.ASA_APPL_PROTO] = "asa_appl_proto";
				descriptions[(System.Int32) IPPorts_Fields.INTRINSA] = "intrinsa";
				descriptions[(System.Int32) IPPorts_Fields.CITADEL] = "citadel";
				descriptions[(System.Int32) IPPorts_Fields.MAILBOX_LM] = "mailbox_lm";
				descriptions[(System.Int32) IPPorts_Fields.OHIMSRV] = "ohimsrv";
				descriptions[(System.Int32) IPPorts_Fields.CRS] = "crs";
				descriptions[(System.Int32) IPPorts_Fields.XVTTP] = "xvttp";
				descriptions[(System.Int32) IPPorts_Fields.SNARE] = "snare";
				descriptions[(System.Int32) IPPorts_Fields.FCP] = "fcp";
				descriptions[(System.Int32) IPPorts_Fields.PASSGO] = "passgo";
				descriptions[(System.Int32) IPPorts_Fields.EXEC] = "exec";
				descriptions[(System.Int32) IPPorts_Fields.LOGIN] = "login";
				descriptions[(System.Int32) IPPorts_Fields.SHELL] = "shell";
				descriptions[(System.Int32) IPPorts_Fields.PRINTER] = "printer";
				descriptions[(System.Int32) IPPorts_Fields.VIDEOTEX] = "videotex";
				descriptions[(System.Int32) IPPorts_Fields.TALK] = "talk";
				descriptions[(System.Int32) IPPorts_Fields.NTALK] = "ntalk";
				descriptions[(System.Int32) IPPorts_Fields.UTIME] = "utime";
				descriptions[(System.Int32) IPPorts_Fields.EFS] = "efs";
				descriptions[(System.Int32) IPPorts_Fields.RIPNG] = "ripng";
				descriptions[(System.Int32) IPPorts_Fields.ULP] = "ulp";
				descriptions[(System.Int32) IPPorts_Fields.IBM_DB2] = "ibm_db2";
				descriptions[(System.Int32) IPPorts_Fields.NCP] = "ncp";
				descriptions[(System.Int32) IPPorts_Fields.TIMED] = "timed";
				descriptions[(System.Int32) IPPorts_Fields.TEMPO] = "tempo";
				descriptions[(System.Int32) IPPorts_Fields.STX] = "stx";
				descriptions[(System.Int32) IPPorts_Fields.CUSTIX] = "custix";
				descriptions[(System.Int32) IPPorts_Fields.IRC_SERV] = "irc_serv";
				descriptions[(System.Int32) IPPorts_Fields.COURIER] = "courier";
				descriptions[(System.Int32) IPPorts_Fields.CONFERENCE] = "conference";
				descriptions[(System.Int32) IPPorts_Fields.NETNEWS] = "netnews";
				descriptions[(System.Int32) IPPorts_Fields.NETWALL] = "netwall";
				descriptions[(System.Int32) IPPorts_Fields.MM_ADMIN] = "mm_admin";
				descriptions[(System.Int32) IPPorts_Fields.IIOP] = "iiop";
				descriptions[(System.Int32) IPPorts_Fields.OPALIS_RDV] = "opalis_rdv";
				descriptions[(System.Int32) IPPorts_Fields.NMSP] = "nmsp";
				descriptions[(System.Int32) IPPorts_Fields.GDOMAP] = "gdomap";
				descriptions[(System.Int32) IPPorts_Fields.APERTUS_LDP] = "apertus_ldp";
				descriptions[(System.Int32) IPPorts_Fields.UUCP] = "uucp";
				descriptions[(System.Int32) IPPorts_Fields.UUCP_RLOGIN] = "uucp_rlogin";
				descriptions[(System.Int32) IPPorts_Fields.COMMERCE] = "commerce";
				descriptions[(System.Int32) IPPorts_Fields.KLOGIN] = "klogin";
				descriptions[(System.Int32) IPPorts_Fields.KSHELL] = "kshell";
				descriptions[(System.Int32) IPPorts_Fields.APPLEQTCSRVR] = "appleqtcsrvr";
				descriptions[(System.Int32) IPPorts_Fields.DHCPV6_CLIENT] = "dhcpv6_client";
				descriptions[(System.Int32) IPPorts_Fields.DHCPV6_SERVER] = "dhcpv6_server";
				descriptions[(System.Int32) IPPorts_Fields.AFPOVERTCP] = "afpovertcp";
				descriptions[(System.Int32) IPPorts_Fields.IDFP] = "idfp";
				descriptions[(System.Int32) IPPorts_Fields.NEW_RWHO] = "new_rwho";
				descriptions[(System.Int32) IPPorts_Fields.CYBERCASH] = "cybercash";
				descriptions[(System.Int32) IPPorts_Fields.DEVICESHARE] = "deviceshare";
				descriptions[(System.Int32) IPPorts_Fields.PIRP] = "pirp";
				descriptions[(System.Int32) IPPorts_Fields.RTSP] = "rtsp";
				descriptions[(System.Int32) IPPorts_Fields.DSF] = "dsf";
				descriptions[(System.Int32) IPPorts_Fields.REMOTEFS] = "remotefs";
				descriptions[(System.Int32) IPPorts_Fields.OPENVMS_SYSIPC] = "openvms_sysipc";
				descriptions[(System.Int32) IPPorts_Fields.SDNSKMP] = "sdnskmp";
				descriptions[(System.Int32) IPPorts_Fields.TEEDTAP] = "teedtap";
				descriptions[(System.Int32) IPPorts_Fields.RMONITOR] = "rmonitor";
				descriptions[(System.Int32) IPPorts_Fields.MONITOR] = "monitor";
				descriptions[(System.Int32) IPPorts_Fields.CHSHELL] = "chshell";
				descriptions[(System.Int32) IPPorts_Fields.NNTPS] = "nntps";
				descriptions[(System.Int32) IPPorts_Fields.N9PFS] = "9pfs";
				descriptions[(System.Int32) IPPorts_Fields.WHOAMI] = "whoami";
				descriptions[(System.Int32) IPPorts_Fields.STREETTALK] = "streettalk";
				descriptions[(System.Int32) IPPorts_Fields.BANYAN_RPC] = "banyan_rpc";
				descriptions[(System.Int32) IPPorts_Fields.MS_SHUTTLE] = "ms_shuttle";
				descriptions[(System.Int32) IPPorts_Fields.MS_ROME] = "ms_rome";
				descriptions[(System.Int32) IPPorts_Fields.METER] = "meter";
				descriptions[(System.Int32) IPPorts_Fields.METER2] = "meter2";
				descriptions[(System.Int32) IPPorts_Fields.SONAR] = "sonar";
				descriptions[(System.Int32) IPPorts_Fields.BANYAN_VIP] = "banyan_vip";
				descriptions[(System.Int32) IPPorts_Fields.FTP_AGENT] = "ftp_agent";
				descriptions[(System.Int32) IPPorts_Fields.VEMMI] = "vemmi";
				descriptions[(System.Int32) IPPorts_Fields.IPCD] = "ipcd";
				descriptions[(System.Int32) IPPorts_Fields.VNAS] = "vnas";
				descriptions[(System.Int32) IPPorts_Fields.IPDD] = "ipdd";
				descriptions[(System.Int32) IPPorts_Fields.DECBSRV] = "decbsrv";
				descriptions[(System.Int32) IPPorts_Fields.SNTP_HEARTBEAT] = "sntp_heartbeat";
				descriptions[(System.Int32) IPPorts_Fields.BDP] = "bdp";
				descriptions[(System.Int32) IPPorts_Fields.SCC_SECURITY] = "scc_security";
				descriptions[(System.Int32) IPPorts_Fields.PHILIPS_VC] = "philips_vc";
				descriptions[(System.Int32) IPPorts_Fields.KEYSERVER] = "keyserver";
				descriptions[(System.Int32) IPPorts_Fields.IMAP4_SSL] = "imap4_ssl";
				descriptions[(System.Int32) IPPorts_Fields.PASSWORD_CHG] = "password_chg";
				descriptions[(System.Int32) IPPorts_Fields.SUBMISSION] = "submission";
				descriptions[(System.Int32) IPPorts_Fields.CAL] = "cal";
				descriptions[(System.Int32) IPPorts_Fields.EYELINK] = "eyelink";
				descriptions[(System.Int32) IPPorts_Fields.TNS_CML] = "tns_cml";
				descriptions[(System.Int32) IPPorts_Fields.HTTP_ALT] = "http_alt";
				descriptions[(System.Int32) IPPorts_Fields.EUDORA_SET] = "eudora_set";
				descriptions[(System.Int32) IPPorts_Fields.HTTP_RPC_EPMAP] = "http_rpc_epmap";
				descriptions[(System.Int32) IPPorts_Fields.TPIP] = "tpip";
				descriptions[(System.Int32) IPPorts_Fields.CAB_PROTOCOL] = "cab_protocol";
				descriptions[(System.Int32) IPPorts_Fields.SMSD] = "smsd";
				descriptions[(System.Int32) IPPorts_Fields.PTCNAMESERVICE] = "ptcnameservice";
				descriptions[(System.Int32) IPPorts_Fields.SCO_WEBSRVRMG3] = "sco_websrvrmg3";
				descriptions[(System.Int32) IPPorts_Fields.ACP] = "acp";
				descriptions[(System.Int32) IPPorts_Fields.IPCSERVER] = "ipcserver";
				descriptions[(System.Int32) IPPorts_Fields.URM] = "urm";
				descriptions[(System.Int32) IPPorts_Fields.NQS] = "nqs";
				descriptions[(System.Int32) IPPorts_Fields.SIFT_UFT] = "sift_uft";
				descriptions[(System.Int32) IPPorts_Fields.NPMP_TRAP] = "npmp_trap";
				descriptions[(System.Int32) IPPorts_Fields.NPMP_LOCAL] = "npmp_local";
				descriptions[(System.Int32) IPPorts_Fields.NPMP_GUI] = "npmp_gui";
				descriptions[(System.Int32) IPPorts_Fields.HMMP_IND] = "hmmp_ind";
				descriptions[(System.Int32) IPPorts_Fields.HMMP_OP] = "hmmp_op";
				descriptions[(System.Int32) IPPorts_Fields.SSHELL] = "sshell";
				descriptions[(System.Int32) IPPorts_Fields.SCO_INETMGR] = "sco_inetmgr";
				descriptions[(System.Int32) IPPorts_Fields.SCO_SYSMGR] = "sco_sysmgr";
				descriptions[(System.Int32) IPPorts_Fields.SCO_DTMGR] = "sco_dtmgr";
				descriptions[(System.Int32) IPPorts_Fields.DEI_ICDA] = "dei_icda";
				descriptions[(System.Int32) IPPorts_Fields.DIGITAL_EVM] = "digital_evm";
				descriptions[(System.Int32) IPPorts_Fields.SCO_WEBSRVRMGR] = "sco_websrvrmgr";
				descriptions[(System.Int32) IPPorts_Fields.ESCP_IP] = "escp_ip";
				descriptions[(System.Int32) IPPorts_Fields.COLLABORATOR] = "collaborator";
				descriptions[(System.Int32) IPPorts_Fields.AUX_BUS_SHUNT] = "aux_bus_shunt";
				descriptions[(System.Int32) IPPorts_Fields.CRYPTOADMIN] = "cryptoadmin";
				descriptions[(System.Int32) IPPorts_Fields.DEC_DLM] = "dec_dlm";
				descriptions[(System.Int32) IPPorts_Fields.ASIA] = "asia";
				descriptions[(System.Int32) IPPorts_Fields.PASSGO_TIVOLI] = "passgo_tivoli";
				descriptions[(System.Int32) IPPorts_Fields.QMQP] = "qmqp";
				descriptions[(System.Int32) IPPorts_Fields.AMP3_3COM] = "3com_amp3";
				descriptions[(System.Int32) IPPorts_Fields.RDA] = "rda";
				descriptions[(System.Int32) IPPorts_Fields.IPP] = "ipp";
				descriptions[(System.Int32) IPPorts_Fields.BMPP] = "bmpp";
				descriptions[(System.Int32) IPPorts_Fields.SERVSTAT] = "servstat";
				descriptions[(System.Int32) IPPorts_Fields.GINAD] = "ginad";
				descriptions[(System.Int32) IPPorts_Fields.RLZDBASE] = "rlzdbase";
				descriptions[(System.Int32) IPPorts_Fields.LDAPS] = "ldaps";
				descriptions[(System.Int32) IPPorts_Fields.LANSERVER] = "lanserver";
				descriptions[(System.Int32) IPPorts_Fields.MCNS_SEC] = "mcns_sec";
				descriptions[(System.Int32) IPPorts_Fields.MSDP] = "msdp";
				descriptions[(System.Int32) IPPorts_Fields.ENTRUST_SPS] = "entrust_sps";
				descriptions[(System.Int32) IPPorts_Fields.REPCMD] = "repcmd";
				descriptions[(System.Int32) IPPorts_Fields.ESRO_EMSDP] = "esro_emsdp";
				descriptions[(System.Int32) IPPorts_Fields.SANITY] = "sanity";
				descriptions[(System.Int32) IPPorts_Fields.DWR] = "dwr";
				descriptions[(System.Int32) IPPorts_Fields.PSSC] = "pssc";
				descriptions[(System.Int32) IPPorts_Fields.LDP] = "ldp";
				descriptions[(System.Int32) IPPorts_Fields.DHCP_FAILOVER] = "dhcp_failover";
				descriptions[(System.Int32) IPPorts_Fields.RRP] = "rrp";
				descriptions[(System.Int32) IPPorts_Fields.AMINET] = "aminet";
				descriptions[(System.Int32) IPPorts_Fields.OBEX] = "obex";
				descriptions[(System.Int32) IPPorts_Fields.IEEE_MMS] = "ieee_mms";
				descriptions[(System.Int32) IPPorts_Fields.HELLO_PORT] = "hello_port";
				descriptions[(System.Int32) IPPorts_Fields.REPSCMD] = "repscmd";
				descriptions[(System.Int32) IPPorts_Fields.AODV] = "aodv";
				descriptions[(System.Int32) IPPorts_Fields.TINC] = "tinc";
				descriptions[(System.Int32) IPPorts_Fields.SPMP] = "spmp";
				descriptions[(System.Int32) IPPorts_Fields.RMC] = "rmc";
				descriptions[(System.Int32) IPPorts_Fields.TENFOLD] = "tenfold";
				descriptions[(System.Int32) IPPorts_Fields.URL_RENDEZVOUS] = "url_rendezvous";
				descriptions[(System.Int32) IPPorts_Fields.MAC_SRVR_ADMIN] = "mac_srvr_admin";
				descriptions[(System.Int32) IPPorts_Fields.HAP] = "hap";
				descriptions[(System.Int32) IPPorts_Fields.PFTP] = "pftp";
				descriptions[(System.Int32) IPPorts_Fields.PURENOISE] = "purenoise";
				descriptions[(System.Int32) IPPorts_Fields.SECURE_AUX_BUS] = "secure_aux_bus";
				descriptions[(System.Int32) IPPorts_Fields.SUN_DR] = "sun_dr";
				descriptions[(System.Int32) IPPorts_Fields.MDQS] = "mdqs";
				descriptions[(System.Int32) IPPorts_Fields.DOOM] = "doom";
				descriptions[(System.Int32) IPPorts_Fields.DISCLOSE] = "disclose";
				descriptions[(System.Int32) IPPorts_Fields.MECOMM] = "mecomm";
				descriptions[(System.Int32) IPPorts_Fields.MEREGISTER] = "meregister";
				descriptions[(System.Int32) IPPorts_Fields.VACDSM_SWS] = "vacdsm_sws";
				descriptions[(System.Int32) IPPorts_Fields.VACDSM_APP] = "vacdsm_app";
				descriptions[(System.Int32) IPPorts_Fields.VPPS_QUA] = "vpps_qua";
				descriptions[(System.Int32) IPPorts_Fields.CIMPLEX] = "cimplex";
				descriptions[(System.Int32) IPPorts_Fields.ACAP] = "acap";
				descriptions[(System.Int32) IPPorts_Fields.DCTP] = "dctp";
				descriptions[(System.Int32) IPPorts_Fields.VPPS_VIA] = "vpps_via";
				descriptions[(System.Int32) IPPorts_Fields.VPP] = "vpp";
				descriptions[(System.Int32) IPPorts_Fields.GGF_NCP] = "ggf_ncp";
				descriptions[(System.Int32) IPPorts_Fields.MRM] = "mrm";
				descriptions[(System.Int32) IPPorts_Fields.ENTRUST_AAAS] = "entrust_aaas";
				descriptions[(System.Int32) IPPorts_Fields.ENTRUST_AAMS] = "entrust_aams";
				descriptions[(System.Int32) IPPorts_Fields.XFR] = "xfr";
				descriptions[(System.Int32) IPPorts_Fields.CORBA_IIOP] = "corba_iiop";
				descriptions[(System.Int32) IPPorts_Fields.CORBA_IIOP_SSL] = "corba_iiop_ssl";
				descriptions[(System.Int32) IPPorts_Fields.MDC_PORTMAPPER] = "mdc_portmapper";
				descriptions[(System.Int32) IPPorts_Fields.HCP_WISMAR] = "hcp_wismar";
				descriptions[(System.Int32) IPPorts_Fields.ASIPREGISTRY] = "asipregistry";
				descriptions[(System.Int32) IPPorts_Fields.REALM_RUSD] = "realm_rusd";
				descriptions[(System.Int32) IPPorts_Fields.NMAP] = "nmap";
				descriptions[(System.Int32) IPPorts_Fields.VATP] = "vatp";
				descriptions[(System.Int32) IPPorts_Fields.MSEXCH_ROUTING] = "msexch_routing";
				descriptions[(System.Int32) IPPorts_Fields.HYPERWAVE_ISP] = "hyperwave_isp";
				descriptions[(System.Int32) IPPorts_Fields.CONNENDP] = "connendp";
				descriptions[(System.Int32) IPPorts_Fields.HA_CLUSTER] = "ha_cluster";
				descriptions[(System.Int32) IPPorts_Fields.IEEE_MMS_SSL] = "ieee_mms_ssl";
				descriptions[(System.Int32) IPPorts_Fields.RUSHD] = "rushd";
				descriptions[(System.Int32) IPPorts_Fields.UUIDGEN] = "uuidgen";
				descriptions[(System.Int32) IPPorts_Fields.OLSR] = "olsr";
				descriptions[(System.Int32) IPPorts_Fields.ACCESSNETWORK] = "accessnetwork";
				descriptions[(System.Int32) IPPorts_Fields.ELCSD] = "elcsd";
				descriptions[(System.Int32) IPPorts_Fields.AGENTX] = "agentx";
				descriptions[(System.Int32) IPPorts_Fields.SILC] = "silc";
				descriptions[(System.Int32) IPPorts_Fields.BORLAND_DSJ] = "borland_dsj";
				descriptions[(System.Int32) IPPorts_Fields.ENTRUST_KMSH] = "entrust_kmsh";
				descriptions[(System.Int32) IPPorts_Fields.ENTRUST_ASH] = "entrust_ash";
				descriptions[(System.Int32) IPPorts_Fields.CISCO_TDP] = "cisco_tdp";
				descriptions[(System.Int32) IPPorts_Fields.NETVIEWDM1] = "netviewdm1";
				descriptions[(System.Int32) IPPorts_Fields.NETVIEWDM2] = "netviewdm2";
				descriptions[(System.Int32) IPPorts_Fields.NETVIEWDM3] = "netviewdm3";
				descriptions[(System.Int32) IPPorts_Fields.NETGW] = "netgw";
				descriptions[(System.Int32) IPPorts_Fields.NETRCS] = "netrcs";
				descriptions[(System.Int32) IPPorts_Fields.FLEXLM] = "flexlm";
				descriptions[(System.Int32) IPPorts_Fields.FUJITSU_DEV] = "fujitsu_dev";
				descriptions[(System.Int32) IPPorts_Fields.RIS_CM] = "ris_cm";
				descriptions[(System.Int32) IPPorts_Fields.KERBEROS_ADM] = "kerberos_adm";
				descriptions[(System.Int32) IPPorts_Fields.RFILE] = "rfile";
				descriptions[(System.Int32) IPPorts_Fields.PUMP] = "pump";
				descriptions[(System.Int32) IPPorts_Fields.QRH] = "qrh";
				descriptions[(System.Int32) IPPorts_Fields.RRH] = "rrh";
				descriptions[(System.Int32) IPPorts_Fields.TELL] = "tell";
				descriptions[(System.Int32) IPPorts_Fields.NLOGIN] = "nlogin";
				descriptions[(System.Int32) IPPorts_Fields.CON] = "con";
				descriptions[(System.Int32) IPPorts_Fields.NS] = "ns";
				descriptions[(System.Int32) IPPorts_Fields.RXE] = "rxe";
				descriptions[(System.Int32) IPPorts_Fields.QUOTAD] = "quotad";
				descriptions[(System.Int32) IPPorts_Fields.CYCLESERV] = "cycleserv";
				descriptions[(System.Int32) IPPorts_Fields.OMSERV] = "omserv";
				descriptions[(System.Int32) IPPorts_Fields.WEBSTER] = "webster";
				descriptions[(System.Int32) IPPorts_Fields.PHONEBOOK] = "phonebook";
				descriptions[(System.Int32) IPPorts_Fields.VID] = "vid";
				descriptions[(System.Int32) IPPorts_Fields.CADLOCK] = "cadlock";
				descriptions[(System.Int32) IPPorts_Fields.RTIP] = "rtip";
				descriptions[(System.Int32) IPPorts_Fields.CYCLESERV2] = "cycleserv2";
				descriptions[(System.Int32) IPPorts_Fields.SUBMIT] = "submit";
				descriptions[(System.Int32) IPPorts_Fields.RPASSWD] = "rpasswd";
				descriptions[(System.Int32) IPPorts_Fields.ENTOMB] = "entomb";
				descriptions[(System.Int32) IPPorts_Fields.WPAGES] = "wpages";
				descriptions[(System.Int32) IPPorts_Fields.WPGS] = "wpgs";
				descriptions[(System.Int32) IPPorts_Fields.QSC] = "qsc";
				descriptions[(System.Int32) IPPorts_Fields.MDBS_DAEMON] = "mdbs_daemon";
				descriptions[(System.Int32) IPPorts_Fields.ITM_MCELL_S] = "itm_mcell_s";
				descriptions[(System.Int32) IPPorts_Fields.PKIX_3_CA_RA] = "pkix_3_ca_ra";
				descriptions[(System.Int32) IPPorts_Fields.DHCP_FAILOVER2] = "dhcp_failover2";
				descriptions[(System.Int32) IPPorts_Fields.RSYNC] = "rsync";
				descriptions[(System.Int32) IPPorts_Fields.ICLCNET_LOCATE] = "iclcnet_locate";
				descriptions[(System.Int32) IPPorts_Fields.ICLCNET_SVINFO] = "iclcnet_svinfo";
				descriptions[(System.Int32) IPPorts_Fields.ACCESSBUILDER] = "accessbuilder";
				descriptions[(System.Int32) IPPorts_Fields.CDDBP] = "cddbp";
				descriptions[(System.Int32) IPPorts_Fields.OMGINITIALREFS] = "omginitialrefs";
				descriptions[(System.Int32) IPPorts_Fields.SMPNAMERES] = "smpnameres";
				descriptions[(System.Int32) IPPorts_Fields.IDEAFARM_CHAT] = "ideafarm_chat";
				descriptions[(System.Int32) IPPorts_Fields.IDEAFARM_CATCH] = "ideafarm_catch";
				descriptions[(System.Int32) IPPorts_Fields.XACT_BACKUP] = "xact_backup";
				descriptions[(System.Int32) IPPorts_Fields.FTPS_DATA] = "ftps_data";
				descriptions[(System.Int32) IPPorts_Fields.FTPS] = "ftps";
				descriptions[(System.Int32) IPPorts_Fields.NAS] = "nas";
				descriptions[(System.Int32) IPPorts_Fields.TELNETS] = "telnets";
				descriptions[(System.Int32) IPPorts_Fields.IMAPS] = "imaps";
				descriptions[(System.Int32) IPPorts_Fields.IRCS] = "ircs";
				descriptions[(System.Int32) IPPorts_Fields.POP3S] = "pop3s";
				descriptions[(System.Int32) IPPorts_Fields.VSINET] = "vsinet";
				descriptions[(System.Int32) IPPorts_Fields.MAITRD] = "maitrd";
				descriptions[(System.Int32) IPPorts_Fields.BUSBOY] = "busboy";
				descriptions[(System.Int32) IPPorts_Fields.GARCON] = "garcon";
				descriptions[(System.Int32) IPPorts_Fields.PUPROUTER] = "puprouter";
				descriptions[(System.Int32) IPPorts_Fields.CADLOCK2] = "cadlock2";
				descriptions[(System.Int32) IPPorts_Fields.SURF] = "surf";
				descriptions[(System.Int32) IPPorts_Fields.RESERVED_1023] = "1023_RESERVED";
				descriptions[(System.Int32) IPPorts_Fields.RESERVED_1024] = "1024_RESERVED";
				descriptions[(System.Int32) IPPorts_Fields.SOCKS] = "socks";
				descriptions[(System.Int32) IPPorts_Fields.HSRP] = "hot standby router";
				descriptions[(System.Int32) IPPorts_Fields.AISES] = "aises";
				descriptions[(System.Int32) IPPorts_Fields.CVS] = "concurrent version system";
				descriptions[(System.Int32) IPPorts_Fields.X11] = "X-Windows";
				
				// todo: table is incomplete
			}
			{
				names[(System.Int32) IPPorts_Fields.NONE] = ".";
				names[(System.Int32) IPPorts_Fields.RESERVED_0] = "0r";
				names[(System.Int32) IPPorts_Fields.TCPMUX] = "tcpmux";
				names[(System.Int32) IPPorts_Fields.COMPRESSNET2] = "compressnet2";
				names[(System.Int32) IPPorts_Fields.COMPRESSNET] = "compressnet";
				names[(System.Int32) IPPorts_Fields.RJE] = "rje";
				names[(System.Int32) IPPorts_Fields.ECHO] = "echo";
				names[(System.Int32) IPPorts_Fields.DISCARD] = "discard";
				names[(System.Int32) IPPorts_Fields.SYSTAT] = "systat";
				names[(System.Int32) IPPorts_Fields.DAYTIME] = "daytime";
				names[(System.Int32) IPPorts_Fields.QOTD] = "qotd";
				names[(System.Int32) IPPorts_Fields.MSP] = "msp";
				names[(System.Int32) IPPorts_Fields.CHARGEN] = "chargen";
				names[(System.Int32) IPPorts_Fields.FTP_DATA] = "ftp_data";
				names[(System.Int32) IPPorts_Fields.FTP] = "ftp";
				names[(System.Int32) IPPorts_Fields.SSH] = "ssh";
				names[(System.Int32) IPPorts_Fields.TELNET] = "telnet";
				names[(System.Int32) IPPorts_Fields.PRIV_MAIL] = "priv_mail";
				names[(System.Int32) IPPorts_Fields.SMTP] = "smtp";
				names[(System.Int32) IPPorts_Fields.NSW_FE] = "nsw_fe";
				names[(System.Int32) IPPorts_Fields.MSG_ICP] = "msg_icp";
				names[(System.Int32) IPPorts_Fields.MSG_AUTH] = "msg_auth";
				names[(System.Int32) IPPorts_Fields.DSP] = "dsp";
				names[(System.Int32) IPPorts_Fields.PRIV_PRINT] = "priv_print";
				names[(System.Int32) IPPorts_Fields.TIME] = "time";
				names[(System.Int32) IPPorts_Fields.RAP] = "rap";
				names[(System.Int32) IPPorts_Fields.RLP] = "rlp";
				names[(System.Int32) IPPorts_Fields.GRAPHICS] = "graphics";
				names[(System.Int32) IPPorts_Fields.NAME] = "name";
				names[(System.Int32) IPPorts_Fields.NAMESERVER] = "nameserver";
				names[(System.Int32) IPPorts_Fields.NICNAME] = "nicname";
				names[(System.Int32) IPPorts_Fields.MPM_FLAGS] = "mpm_flags";
				names[(System.Int32) IPPorts_Fields.MPM] = "mpm";
				names[(System.Int32) IPPorts_Fields.MPM_SND] = "mpm_snd";
				names[(System.Int32) IPPorts_Fields.NI_FTP] = "ni_ftp";
				names[(System.Int32) IPPorts_Fields.AUDITD] = "auditd";
				names[(System.Int32) IPPorts_Fields.TACACS] = "tacacs";
				names[(System.Int32) IPPorts_Fields.RE_MAIL_CK] = "re_mail_ck";
				names[(System.Int32) IPPorts_Fields.LA_MAINT] = "la_maint";
				names[(System.Int32) IPPorts_Fields.XNS_TIME] = "xns_time";
				names[(System.Int32) IPPorts_Fields.DOMAIN] = "domain";
				names[(System.Int32) IPPorts_Fields.XNS_CH] = "xns_ch";
				names[(System.Int32) IPPorts_Fields.ISI_GL] = "isi_gl";
				names[(System.Int32) IPPorts_Fields.XNS_AUTH] = "xns_auth";
				names[(System.Int32) IPPorts_Fields.PRIV_TERMINAL] = "priv_terminal";
				names[(System.Int32) IPPorts_Fields.XNS_MAIL] = "xns_mail";
				names[(System.Int32) IPPorts_Fields.PRIV_FILE] = "priv_file";
				names[(System.Int32) IPPorts_Fields.NI_MAIL] = "ni_mail";
				names[(System.Int32) IPPorts_Fields.ACAS] = "acas";
				names[(System.Int32) IPPorts_Fields.WHOIS] = "whois++";
				names[(System.Int32) IPPorts_Fields.COVIA] = "covia";
				names[(System.Int32) IPPorts_Fields.TACACS_DS] = "tacacs_ds";
				names[(System.Int32) IPPorts_Fields.SQLSNET] = "sql*net";
				names[(System.Int32) IPPorts_Fields.BOOTPS] = "bootps";
				names[(System.Int32) IPPorts_Fields.BOOTPC] = "bootpc";
				names[(System.Int32) IPPorts_Fields.TFTP] = "tftp";
				names[(System.Int32) IPPorts_Fields.GOPHER] = "gopher";
				names[(System.Int32) IPPorts_Fields.NETRJS_1] = "netrjs_1";
				names[(System.Int32) IPPorts_Fields.NETRJS_2] = "netrjs_2";
				names[(System.Int32) IPPorts_Fields.NETRJS_3] = "netrjs_3";
				names[(System.Int32) IPPorts_Fields.NETRJS_4] = "netrjs_4";
				names[(System.Int32) IPPorts_Fields.PRIV_DIALOUT] = "priv_dialout";
				names[(System.Int32) IPPorts_Fields.DEOS] = "deos";
				names[(System.Int32) IPPorts_Fields.PRIV_RJE] = "priv_rje";
				names[(System.Int32) IPPorts_Fields.VETTCP] = "vettcp";
				names[(System.Int32) IPPorts_Fields.FINGER] = "finger";
				names[(System.Int32) IPPorts_Fields.HTTP] = "http";
				names[(System.Int32) IPPorts_Fields.WWW] = "www";
				names[(System.Int32) IPPorts_Fields.HOSTS2_NS] = "hosts2_ns";
				names[(System.Int32) IPPorts_Fields.XFER] = "xfer";
				names[(System.Int32) IPPorts_Fields.MIT_ML_DEV] = "mit_ml_dev";
				names[(System.Int32) IPPorts_Fields.CTF] = "ctf";
				names[(System.Int32) IPPorts_Fields.MIT_ML_DEV2] = "mit_ml_dev";
				names[(System.Int32) IPPorts_Fields.MFCOBOL] = "mfcobol";
				names[(System.Int32) IPPorts_Fields.PRIV_TERMLINK] = "priv_termlink";
				names[(System.Int32) IPPorts_Fields.KERBEROS] = "kerberos";
				names[(System.Int32) IPPorts_Fields.SU_MIT_TG] = "su_mit_tg";
				names[(System.Int32) IPPorts_Fields.DNSIX] = "dnsix";
				names[(System.Int32) IPPorts_Fields.MIT_DOV] = "mit_dov";
				names[(System.Int32) IPPorts_Fields.NPP] = "npp";
				names[(System.Int32) IPPorts_Fields.DCP] = "dcp";
				names[(System.Int32) IPPorts_Fields.OBJCALL] = "objcall";
				names[(System.Int32) IPPorts_Fields.SUPDUP] = "supdup";
				names[(System.Int32) IPPorts_Fields.DIXIE] = "dixie";
				names[(System.Int32) IPPorts_Fields.SWIFT_RVF] = "swift_rvf";
				names[(System.Int32) IPPorts_Fields.TACNEWS] = "tacnews";
				names[(System.Int32) IPPorts_Fields.METAGRAM] = "metagram";
				names[(System.Int32) IPPorts_Fields.NEWACCT] = "newacct";
				names[(System.Int32) IPPorts_Fields.HOSTNAME] = "hostname";
				names[(System.Int32) IPPorts_Fields.ISO_TSAP] = "iso_tsap";
				names[(System.Int32) IPPorts_Fields.GPPITNP] = "gppitnp";
				names[(System.Int32) IPPorts_Fields.ACR_NEMA] = "acr_nema";
				names[(System.Int32) IPPorts_Fields.CSO] = "cso";
				names[(System.Int32) IPPorts_Fields.CSNET_NS] = "csnet_ns";
				names[(System.Int32) IPPorts_Fields.TSMUX_3COM] = "3com_tsmux";
				names[(System.Int32) IPPorts_Fields.RTELNET] = "rtelnet";
				names[(System.Int32) IPPorts_Fields.SNAGAS] = "snagas";
				names[(System.Int32) IPPorts_Fields.POP2] = "pop2";
				names[(System.Int32) IPPorts_Fields.POP3] = "pop3";
				names[(System.Int32) IPPorts_Fields.SUNRPC] = "sunrpc";
				names[(System.Int32) IPPorts_Fields.MCIDAS] = "mcidas";
				names[(System.Int32) IPPorts_Fields.IDENT] = "ident";
				names[(System.Int32) IPPorts_Fields.AUTH] = "auth";
				names[(System.Int32) IPPorts_Fields.AUDIONEWS] = "audionews";
				names[(System.Int32) IPPorts_Fields.SFTP] = "sftp";
				names[(System.Int32) IPPorts_Fields.ANSANOTIFY] = "ansanotify";
				names[(System.Int32) IPPorts_Fields.UUCP_PATH] = "uucp_path";
				names[(System.Int32) IPPorts_Fields.SQLSERV] = "sqlserv";
				names[(System.Int32) IPPorts_Fields.NNTP] = "nntp";
				names[(System.Int32) IPPorts_Fields.CFDPTKT] = "cfdptkt";
				names[(System.Int32) IPPorts_Fields.ERPC] = "erpc";
				names[(System.Int32) IPPorts_Fields.SMAKYNET] = "smakynet";
				names[(System.Int32) IPPorts_Fields.NTP] = "ntp";
				names[(System.Int32) IPPorts_Fields.ANSATRADER] = "ansatrader";
				names[(System.Int32) IPPorts_Fields.LOCUS_MAP] = "locus_map";
				names[(System.Int32) IPPorts_Fields.NXEDIT] = "nxedit";
				names[(System.Int32) IPPorts_Fields.LOCUS_CON] = "locus_con";
				names[(System.Int32) IPPorts_Fields.GSS_XLICEN] = "gss_xlicen";
				names[(System.Int32) IPPorts_Fields.PWDGEN] = "pwdgen";
				names[(System.Int32) IPPorts_Fields.CISCO_FNA] = "cisco_fna";
				names[(System.Int32) IPPorts_Fields.CISCO_TNA] = "cisco_tna";
				names[(System.Int32) IPPorts_Fields.CISCO_SYS] = "cisco_sys";
				names[(System.Int32) IPPorts_Fields.STATSRV] = "statsrv";
				names[(System.Int32) IPPorts_Fields.INGRES_NET] = "ingres_net";
				names[(System.Int32) IPPorts_Fields.EPMAP] = "epmap";
				names[(System.Int32) IPPorts_Fields.PROFILE] = "profile";
				names[(System.Int32) IPPorts_Fields.NETBIOS_NS] = "netbios_ns";
				names[(System.Int32) IPPorts_Fields.NETBIOS_DGM] = "netbios_dgm";
				names[(System.Int32) IPPorts_Fields.NETBIOS_SSN] = "netbios_ssn";
				names[(System.Int32) IPPorts_Fields.EMFIS_DATA] = "emfis_data";
				names[(System.Int32) IPPorts_Fields.EMFIS_CNTL] = "emfis_cntl";
				names[(System.Int32) IPPorts_Fields.BL_IDM] = "bl_idm";
				names[(System.Int32) IPPorts_Fields.IMAP] = "imap";
				names[(System.Int32) IPPorts_Fields.UMA] = "uma";
				names[(System.Int32) IPPorts_Fields.UAAC] = "uaac";
				names[(System.Int32) IPPorts_Fields.ISO_TP0] = "iso_tp0";
				names[(System.Int32) IPPorts_Fields.ISO_IP] = "iso_ip";
				names[(System.Int32) IPPorts_Fields.JARGON] = "jargon";
				names[(System.Int32) IPPorts_Fields.AED_512] = "aed_512";
				names[(System.Int32) IPPorts_Fields.SQL_NET] = "sql_net";
				names[(System.Int32) IPPorts_Fields.HEMS] = "hems";
				names[(System.Int32) IPPorts_Fields.BFTP] = "bftp";
				names[(System.Int32) IPPorts_Fields.SGMP] = "sgmp";
				names[(System.Int32) IPPorts_Fields.NETSC_PROD] = "netsc_prod";
				names[(System.Int32) IPPorts_Fields.NETSC_DEV] = "netsc_dev";
				names[(System.Int32) IPPorts_Fields.SQLSRV] = "sqlsrv";
				names[(System.Int32) IPPorts_Fields.KNET_CMP] = "knet_cmp";
				names[(System.Int32) IPPorts_Fields.PCMAIL_SRV] = "pcmail_srv";
				names[(System.Int32) IPPorts_Fields.NSS_ROUTING] = "nss_routing";
				names[(System.Int32) IPPorts_Fields.SGMP_TRAPS] = "sgmp_traps";
				names[(System.Int32) IPPorts_Fields.SNMP] = "snmp";
				names[(System.Int32) IPPorts_Fields.SNMPTRAP] = "snmptrap";
				names[(System.Int32) IPPorts_Fields.CMIP_MAN] = "cmip_man";
				names[(System.Int32) IPPorts_Fields.CMIP_AGENT] = "cmip_agent";
				names[(System.Int32) IPPorts_Fields.XNS_COURIER] = "xns_courier";
				names[(System.Int32) IPPorts_Fields.S_NET] = "s_net";
				names[(System.Int32) IPPorts_Fields.NAMP] = "namp";
				names[(System.Int32) IPPorts_Fields.RSVD] = "rsvd";
				names[(System.Int32) IPPorts_Fields.SEND] = "send";
				names[(System.Int32) IPPorts_Fields.PRINT_SRV] = "print_srv";
				names[(System.Int32) IPPorts_Fields.MULTIPLEX] = "multiplex";
				names[(System.Int32) IPPorts_Fields.CL1] = "cl1";
				names[(System.Int32) IPPorts_Fields.XYPLEX_MUX] = "xyplex_mux";
				names[(System.Int32) IPPorts_Fields.MAILQ] = "mailq";
				names[(System.Int32) IPPorts_Fields.VMNET] = "vmnet";
				names[(System.Int32) IPPorts_Fields.GENRAD_MUX] = "genrad_mux";
				names[(System.Int32) IPPorts_Fields.XDMCP] = "xdmcp";
				names[(System.Int32) IPPorts_Fields.NEXTSTEP] = "nextstep";
				names[(System.Int32) IPPorts_Fields.BGP] = "bgp";
				names[(System.Int32) IPPorts_Fields.RIS] = "ris";
				names[(System.Int32) IPPorts_Fields.UNIFY] = "unify";
				names[(System.Int32) IPPorts_Fields.AUDIT] = "audit";
				names[(System.Int32) IPPorts_Fields.OCBINDER] = "ocbinder";
				names[(System.Int32) IPPorts_Fields.OCSERVER] = "ocserver";
				names[(System.Int32) IPPorts_Fields.REMOTE_KIS] = "remote_kis";
				names[(System.Int32) IPPorts_Fields.KIS] = "kis";
				names[(System.Int32) IPPorts_Fields.ACI] = "aci";
				names[(System.Int32) IPPorts_Fields.MUMPS] = "mumps";
				names[(System.Int32) IPPorts_Fields.QFT] = "qft";
				names[(System.Int32) IPPorts_Fields.GACP] = "gacp";
				names[(System.Int32) IPPorts_Fields.PROSPERO] = "prospero";
				names[(System.Int32) IPPorts_Fields.OSU_NMS] = "osu_nms";
				names[(System.Int32) IPPorts_Fields.SRMP] = "srmp";
				names[(System.Int32) IPPorts_Fields.IRC] = "irc";
				names[(System.Int32) IPPorts_Fields.DN6_NLM_AUD] = "dn6_nlm_aud";
				names[(System.Int32) IPPorts_Fields.DN6_SMM_RED] = "dn6_smm_red";
				names[(System.Int32) IPPorts_Fields.DLS] = "dls";
				names[(System.Int32) IPPorts_Fields.DLS_MON] = "dls_mon";
				names[(System.Int32) IPPorts_Fields.SMUX] = "smux";
				names[(System.Int32) IPPorts_Fields.SRC] = "src";
				names[(System.Int32) IPPorts_Fields.AT_RTMP] = "at_rtmp";
				names[(System.Int32) IPPorts_Fields.AT_NBP] = "at_nbp";
				names[(System.Int32) IPPorts_Fields.AT_3] = "at_3";
				names[(System.Int32) IPPorts_Fields.AT_ECHO] = "at_echo";
				names[(System.Int32) IPPorts_Fields.AT_5] = "at_5";
				names[(System.Int32) IPPorts_Fields.AT_ZIS] = "at_zis";
				names[(System.Int32) IPPorts_Fields.AT_7] = "at_7";
				names[(System.Int32) IPPorts_Fields.AT_8] = "at_8";
				names[(System.Int32) IPPorts_Fields.QMTP] = "qmtp";
				names[(System.Int32) IPPorts_Fields.Z39_50] = "z39.50";
				names[(System.Int32) IPPorts_Fields.N914C] = "914c_g";
				names[(System.Int32) IPPorts_Fields.ANET] = "anet";
				names[(System.Int32) IPPorts_Fields.IPX] = "ipx";
				names[(System.Int32) IPPorts_Fields.VMPWSCS] = "vmpwscs";
				names[(System.Int32) IPPorts_Fields.SOFTPC] = "softpc";
				names[(System.Int32) IPPorts_Fields.CAILIC] = "CAIlic";
				names[(System.Int32) IPPorts_Fields.DBASE] = "dbase";
				names[(System.Int32) IPPorts_Fields.MPP] = "mpp";
				names[(System.Int32) IPPorts_Fields.UARPS] = "uarps";
				names[(System.Int32) IPPorts_Fields.IMAP3] = "imap3";
				names[(System.Int32) IPPorts_Fields.FLN_SPX] = "fln_spx";
				names[(System.Int32) IPPorts_Fields.RSH_SPX] = "rsh_spx";
				names[(System.Int32) IPPorts_Fields.CDC] = "cdc";
				names[(System.Int32) IPPorts_Fields.MASQDIALER] = "masqdialer";
				names[(System.Int32) IPPorts_Fields.DIRECT] = "direct";
				names[(System.Int32) IPPorts_Fields.SUR_MEAS] = "sur_meas";
				names[(System.Int32) IPPorts_Fields.INBUSINESS] = "inbusiness";
				names[(System.Int32) IPPorts_Fields.LINK] = "link";
				names[(System.Int32) IPPorts_Fields.DSP3270] = "dsp3270";
				names[(System.Int32) IPPorts_Fields.SUBNTBCST_TFTP] = "subntbcst_tftp";
				names[(System.Int32) IPPorts_Fields.BHFHS] = "bhfhs";
				names[(System.Int32) IPPorts_Fields.RAP2] = "rap2";
				names[(System.Int32) IPPorts_Fields.SET] = "set";
				names[(System.Int32) IPPorts_Fields.YAK_CHAT] = "yak_chat";
				names[(System.Int32) IPPorts_Fields.ESRO_GEN] = "esro_gen";
				names[(System.Int32) IPPorts_Fields.OPENPORT] = "openport";
				names[(System.Int32) IPPorts_Fields.NSIIOPS] = "nsiiops";
				names[(System.Int32) IPPorts_Fields.ARCISDMS] = "arcisdms";
				names[(System.Int32) IPPorts_Fields.HDAP] = "hdap";
				names[(System.Int32) IPPorts_Fields.BGMP] = "bgmp";
				names[(System.Int32) IPPorts_Fields.X_BONE_CTL] = "x_bone_ctl";
				names[(System.Int32) IPPorts_Fields.SST] = "sst";
				names[(System.Int32) IPPorts_Fields.TD_SERVICE] = "td_service";
				names[(System.Int32) IPPorts_Fields.TD_REPLICA] = "td_replica";
				names[(System.Int32) IPPorts_Fields.HTTP_MGMT] = "http_mgmt";
				names[(System.Int32) IPPorts_Fields.PERSONAL_LINK] = "personal_link";
				names[(System.Int32) IPPorts_Fields.CABLEPORT_AX] = "cableport_ax";
				names[(System.Int32) IPPorts_Fields.RESCAP] = "rescap";
				names[(System.Int32) IPPorts_Fields.CORERJD] = "corerjd";
				names[(System.Int32) IPPorts_Fields.FXP_1] = "fxp_1";
				names[(System.Int32) IPPorts_Fields.K_BLOCK] = "k_block";
				names[(System.Int32) IPPorts_Fields.NOVASTORBAKCUP] = "novastorbakcup";
				names[(System.Int32) IPPorts_Fields.ENTRUSTTIME] = "entrusttime";
				names[(System.Int32) IPPorts_Fields.BHMDS] = "bhmds";
				names[(System.Int32) IPPorts_Fields.ASIP_WEBADMIN] = "asip_webadmin";
				names[(System.Int32) IPPorts_Fields.VSLMP] = "vslmp";
				names[(System.Int32) IPPorts_Fields.MAGENTA_LOGIC] = "magenta_logic";
				names[(System.Int32) IPPorts_Fields.OPALIS_ROBOT] = "opalis_robot";
				names[(System.Int32) IPPorts_Fields.DPSI] = "dpsi";
				names[(System.Int32) IPPorts_Fields.DECAUTH] = "decauth";
				names[(System.Int32) IPPorts_Fields.ZANNET] = "zannet";
				names[(System.Int32) IPPorts_Fields.PKIX_TIMESTAMP] = "pkix_timestamp";
				names[(System.Int32) IPPorts_Fields.PTP_EVENT] = "ptp_event";
				names[(System.Int32) IPPorts_Fields.PTP_GENERAL] = "ptp_general";
				names[(System.Int32) IPPorts_Fields.PIP] = "pip";
				names[(System.Int32) IPPorts_Fields.RTSPS] = "rtsps";
				names[(System.Int32) IPPorts_Fields.TEXAR] = "texar";
				names[(System.Int32) IPPorts_Fields.PDAP] = "pdap";
				names[(System.Int32) IPPorts_Fields.PAWSERV] = "pawserv";
				names[(System.Int32) IPPorts_Fields.ZSERV] = "zserv";
				names[(System.Int32) IPPorts_Fields.FATSERV] = "fatserv";
				names[(System.Int32) IPPorts_Fields.CSI_SGWP] = "csi_sgwp";
				names[(System.Int32) IPPorts_Fields.MFTP] = "mftp";
				names[(System.Int32) IPPorts_Fields.MATIP_TYPE_A] = "matip_type_a";
				names[(System.Int32) IPPorts_Fields.MATIP_TYPE_B] = "matip_type_b";
				names[(System.Int32) IPPorts_Fields.BHOETTY] = "bhoetty";
				names[(System.Int32) IPPorts_Fields.DTAG_STE_SB] = "dtag_ste_sb";
				names[(System.Int32) IPPorts_Fields.BHOEDAP4] = "bhoedap4";
				names[(System.Int32) IPPorts_Fields.NDSAUTH] = "ndsauth";
				names[(System.Int32) IPPorts_Fields.BH611] = "bh611";
				names[(System.Int32) IPPorts_Fields.DATEX_ASN] = "datex_asn";
				names[(System.Int32) IPPorts_Fields.CLOANTO_NET] = "cloanto_net";
				names[(System.Int32) IPPorts_Fields.BHEVENT] = "bhevent";
				names[(System.Int32) IPPorts_Fields.SHRINKWRAP] = "shrinkwrap";
				names[(System.Int32) IPPorts_Fields.NSRMP] = "nsrmp";
				names[(System.Int32) IPPorts_Fields.SCOI2ODIALOG] = "scoi2odialog";
				names[(System.Int32) IPPorts_Fields.SEMANTIX] = "semantix";
				names[(System.Int32) IPPorts_Fields.SRSSEND] = "srssend";
				names[(System.Int32) IPPorts_Fields.RSVP_TUNNEL] = "rsvp_tunnel";
				names[(System.Int32) IPPorts_Fields.AURORA_CMGR] = "aurora_cmgr";
				names[(System.Int32) IPPorts_Fields.DTK] = "dtk";
				names[(System.Int32) IPPorts_Fields.ODMR] = "odmr";
				names[(System.Int32) IPPorts_Fields.MORTGAGEWARE] = "mortgageware";
				names[(System.Int32) IPPorts_Fields.QBIKGDP] = "qbikgdp";
				names[(System.Int32) IPPorts_Fields.RPC2PORTMAP] = "rpc2portmap";
				names[(System.Int32) IPPorts_Fields.CODAAUTH] = "codaauth";
				names[(System.Int32) IPPorts_Fields.CLEARCASE] = "clearcase";
				names[(System.Int32) IPPorts_Fields.ULISTPROC] = "ulistproc";
				names[(System.Int32) IPPorts_Fields.LEGENT_1] = "legent_1";
				names[(System.Int32) IPPorts_Fields.LEGENT_2] = "legent_2";
				names[(System.Int32) IPPorts_Fields.HASSLE] = "hassle";
				names[(System.Int32) IPPorts_Fields.NIP] = "nip";
				names[(System.Int32) IPPorts_Fields.TNETOS] = "tnETOS";
				names[(System.Int32) IPPorts_Fields.DSETOS] = "dsETOS";
				names[(System.Int32) IPPorts_Fields.IS99C] = "is99c";
				names[(System.Int32) IPPorts_Fields.IS99S] = "is99s";
				names[(System.Int32) IPPorts_Fields.HP_COLLECTOR] = "hp_collector";
				names[(System.Int32) IPPorts_Fields.HP_MANAGED_NODE] = "hp_managed_node";
				names[(System.Int32) IPPorts_Fields.HP_ALARM_MGR] = "hp_alarm_mgr";
				names[(System.Int32) IPPorts_Fields.ARNS] = "arns";
				names[(System.Int32) IPPorts_Fields.IBM_APP] = "ibm_app";
				names[(System.Int32) IPPorts_Fields.ASA] = "asa";
				names[(System.Int32) IPPorts_Fields.AURP] = "aurp";
				names[(System.Int32) IPPorts_Fields.UNIDATA_LDM] = "unidata_ldm";
				names[(System.Int32) IPPorts_Fields.UIS] = "uis";
				names[(System.Int32) IPPorts_Fields.SYNOTICS_RELAY] = "synotics_relay";
				names[(System.Int32) IPPorts_Fields.SYNOTICS_BROKER] = "synotics_broker";
				names[(System.Int32) IPPorts_Fields.META5] = "meta5";
				names[(System.Int32) IPPorts_Fields.EMBL_NDT] = "embl_ndt";
				names[(System.Int32) IPPorts_Fields.NETCP] = "netcp";
				names[(System.Int32) IPPorts_Fields.NETWARE_IP] = "netware_ip";
				names[(System.Int32) IPPorts_Fields.MPTN] = "mptn";
				names[(System.Int32) IPPorts_Fields.KRYPTOLAN] = "kryptolan";
				names[(System.Int32) IPPorts_Fields.ISO_TSAP_C2] = "iso_tsap_c2";
				names[(System.Int32) IPPorts_Fields.WORK_SOL] = "work_sol";
				names[(System.Int32) IPPorts_Fields.UPS] = "ups";
				names[(System.Int32) IPPorts_Fields.GENIE] = "genie";
				names[(System.Int32) IPPorts_Fields.DECAP] = "decap";
				names[(System.Int32) IPPorts_Fields.NCED] = "nced";
				names[(System.Int32) IPPorts_Fields.NCLD] = "ncld";
				names[(System.Int32) IPPorts_Fields.IMSP] = "imsp";
				names[(System.Int32) IPPorts_Fields.TIMBUKTU] = "timbuktu";
				names[(System.Int32) IPPorts_Fields.PRM_SM] = "prm_sm";
				names[(System.Int32) IPPorts_Fields.PRM_NM] = "prm_nm";
				names[(System.Int32) IPPorts_Fields.DECLADEBUG] = "decladebug";
				names[(System.Int32) IPPorts_Fields.RMT] = "rmt";
				names[(System.Int32) IPPorts_Fields.SYNOPTICS_TRAP] = "synoptics_trap";
				names[(System.Int32) IPPorts_Fields.SMSP] = "smsp";
				names[(System.Int32) IPPorts_Fields.INFOSEEK] = "infoseek";
				names[(System.Int32) IPPorts_Fields.BNET] = "bnet";
				names[(System.Int32) IPPorts_Fields.SILVERPLATTER] = "silverplatter";
				names[(System.Int32) IPPorts_Fields.ONMUX] = "onmux";
				names[(System.Int32) IPPorts_Fields.HYPER_G] = "hyper_g";
				names[(System.Int32) IPPorts_Fields.ARIEL1] = "ariel1";
				names[(System.Int32) IPPorts_Fields.SMPTE] = "smpte";
				names[(System.Int32) IPPorts_Fields.ARIEL2] = "ariel2";
				names[(System.Int32) IPPorts_Fields.ARIEL3] = "ariel3";
				names[(System.Int32) IPPorts_Fields.OPC_JOB_START] = "opc_job_start";
				names[(System.Int32) IPPorts_Fields.OPC_JOB_TRACK] = "opc_job_track";
				names[(System.Int32) IPPorts_Fields.ICAD_EL] = "icad_el";
				names[(System.Int32) IPPorts_Fields.SMARTSDP] = "smartsdp";
				names[(System.Int32) IPPorts_Fields.SVRLOC] = "svrloc";
				names[(System.Int32) IPPorts_Fields.OCS_CMU] = "ocs_cmu";
				names[(System.Int32) IPPorts_Fields.OCS_AMU] = "ocs_amu";
				names[(System.Int32) IPPorts_Fields.UTMPSD] = "utmpsd";
				names[(System.Int32) IPPorts_Fields.UTMPCD] = "utmpcd";
				names[(System.Int32) IPPorts_Fields.IASD] = "iasd";
				names[(System.Int32) IPPorts_Fields.NNSP] = "nnsp";
				names[(System.Int32) IPPorts_Fields.MOBILEIP_AGENT] = "mobileip_agent";
				names[(System.Int32) IPPorts_Fields.MOBILIP_MN] = "mobilip_mn";
				names[(System.Int32) IPPorts_Fields.DNA_CML] = "dna_cml";
				names[(System.Int32) IPPorts_Fields.COMSCM] = "comscm";
				names[(System.Int32) IPPorts_Fields.DSFGW] = "dsfgw";
				names[(System.Int32) IPPorts_Fields.DASP] = "dasp";
				names[(System.Int32) IPPorts_Fields.SGCP] = "sgcp";
				names[(System.Int32) IPPorts_Fields.DECVMS_SYSMGT] = "decvms_sysmgt";
				names[(System.Int32) IPPorts_Fields.CVC_HOSTD] = "cvc_hostd";
				names[(System.Int32) IPPorts_Fields.HTTPS] = "https";
				names[(System.Int32) IPPorts_Fields.SNPP] = "snpp";
				names[(System.Int32) IPPorts_Fields.MICROSOFT_DS] = "microsoft_ds";
				names[(System.Int32) IPPorts_Fields.DDM_RDB] = "ddm_rdb";
				names[(System.Int32) IPPorts_Fields.DDM_DFM] = "ddm_dfm";
				names[(System.Int32) IPPorts_Fields.DDM_SSL] = "ddm_ssl";
				names[(System.Int32) IPPorts_Fields.AS_SERVERMAP] = "as_servermap";
				names[(System.Int32) IPPorts_Fields.TSERVER] = "tserver";
				names[(System.Int32) IPPorts_Fields.SFS_SMP_NET] = "sfs_smp_net";
				names[(System.Int32) IPPorts_Fields.SFS_CONFIG] = "sfs_config";
				names[(System.Int32) IPPorts_Fields.CREATIVESERVER] = "creativeserver";
				names[(System.Int32) IPPorts_Fields.CONTENTSERVER] = "contentserver";
				names[(System.Int32) IPPorts_Fields.CREATIVEPARTNR] = "creativepartnr";
				names[(System.Int32) IPPorts_Fields.MACON_TCP] = "macon_tcp";
				names[(System.Int32) IPPorts_Fields.SCOHELP] = "scohelp";
				names[(System.Int32) IPPorts_Fields.APPLEQTC] = "appleqtc";
				names[(System.Int32) IPPorts_Fields.AMPR_RCMD] = "ampr_rcmd";
				names[(System.Int32) IPPorts_Fields.SKRONK] = "skronk";
				names[(System.Int32) IPPorts_Fields.DATASURFSRV] = "datasurfsrv";
				names[(System.Int32) IPPorts_Fields.DATASURFSRVSEC] = "datasurfsrvsec";
				names[(System.Int32) IPPorts_Fields.ALPES] = "alpes";
				names[(System.Int32) IPPorts_Fields.KPASSWD] = "kpasswd";
				names[(System.Int32) IPPorts_Fields.DIGITAL_VRC] = "digital_vrc";
				names[(System.Int32) IPPorts_Fields.MYLEX_MAPD] = "mylex_mapd";
				names[(System.Int32) IPPorts_Fields.PHOTURIS] = "photuris";
				names[(System.Int32) IPPorts_Fields.RCP] = "rcp";
				names[(System.Int32) IPPorts_Fields.SCX_PROXY] = "scx_proxy";
				names[(System.Int32) IPPorts_Fields.MONDEX] = "mondex";
				names[(System.Int32) IPPorts_Fields.LJK_LOGIN] = "ljk_login";
				names[(System.Int32) IPPorts_Fields.HYBRID_POP] = "hybrid_pop";
				names[(System.Int32) IPPorts_Fields.TN_TL_W1] = "tn_tl_w1";
				names[(System.Int32) IPPorts_Fields.TCPNETHASPSRV] = "tcpnethaspsrv";
				names[(System.Int32) IPPorts_Fields.TN_TL_FD1] = "tn_tl_fd1";
				names[(System.Int32) IPPorts_Fields.SS7NS] = "ss7ns";
				names[(System.Int32) IPPorts_Fields.SPSC] = "spsc";
				names[(System.Int32) IPPorts_Fields.IAFSERVER] = "iafserver";
				names[(System.Int32) IPPorts_Fields.IAFDBASE] = "iafdbase";
				names[(System.Int32) IPPorts_Fields.PH] = "ph";
				names[(System.Int32) IPPorts_Fields.BGS_NSI] = "bgs_nsi";
				names[(System.Int32) IPPorts_Fields.ULPNET] = "ulpnet";
				names[(System.Int32) IPPorts_Fields.INTEGRA_SME] = "integra_sme";
				names[(System.Int32) IPPorts_Fields.POWERBURST] = "powerburst";
				names[(System.Int32) IPPorts_Fields.AVIAN] = "avian";
				names[(System.Int32) IPPorts_Fields.SAFT] = "saft";
				names[(System.Int32) IPPorts_Fields.GSS_HTTP] = "gss_http";
				names[(System.Int32) IPPorts_Fields.NEST_PROTOCOL] = "nest_protocol";
				names[(System.Int32) IPPorts_Fields.MICOM_PFS] = "micom_pfs";
				names[(System.Int32) IPPorts_Fields.GO_LOGIN] = "go_login";
				names[(System.Int32) IPPorts_Fields.TICF_1] = "ticf_1";
				names[(System.Int32) IPPorts_Fields.TICF_2] = "ticf_2";
				names[(System.Int32) IPPorts_Fields.POV_RAY] = "pov_ray";
				names[(System.Int32) IPPorts_Fields.INTECOURIER] = "intecourier";
				names[(System.Int32) IPPorts_Fields.PIM_RP_DISC] = "pim_rp_disc";
				names[(System.Int32) IPPorts_Fields.DANTZ] = "dantz";
				names[(System.Int32) IPPorts_Fields.SIAM] = "siam";
				names[(System.Int32) IPPorts_Fields.ISO_ILL] = "iso_ill";
				names[(System.Int32) IPPorts_Fields.ISAKMP] = "isakmp";
				names[(System.Int32) IPPorts_Fields.STMF] = "stmf";
				names[(System.Int32) IPPorts_Fields.ASA_APPL_PROTO] = "asa_appl_proto";
				names[(System.Int32) IPPorts_Fields.INTRINSA] = "intrinsa";
				names[(System.Int32) IPPorts_Fields.CITADEL] = "citadel";
				names[(System.Int32) IPPorts_Fields.MAILBOX_LM] = "mailbox_lm";
				names[(System.Int32) IPPorts_Fields.OHIMSRV] = "ohimsrv";
				names[(System.Int32) IPPorts_Fields.CRS] = "crs";
				names[(System.Int32) IPPorts_Fields.XVTTP] = "xvttp";
				names[(System.Int32) IPPorts_Fields.SNARE] = "snare";
				names[(System.Int32) IPPorts_Fields.FCP] = "fcp";
				names[(System.Int32) IPPorts_Fields.PASSGO] = "passgo";
				names[(System.Int32) IPPorts_Fields.EXEC] = "exec";
				names[(System.Int32) IPPorts_Fields.LOGIN] = "login";
				names[(System.Int32) IPPorts_Fields.SHELL] = "shell";
				names[(System.Int32) IPPorts_Fields.PRINTER] = "printer";
				names[(System.Int32) IPPorts_Fields.VIDEOTEX] = "videotex";
				names[(System.Int32) IPPorts_Fields.TALK] = "talk";
				names[(System.Int32) IPPorts_Fields.NTALK] = "ntalk";
				names[(System.Int32) IPPorts_Fields.UTIME] = "utime";
				names[(System.Int32) IPPorts_Fields.EFS] = "efs";
				names[(System.Int32) IPPorts_Fields.RIPNG] = "ripng";
				names[(System.Int32) IPPorts_Fields.ULP] = "ulp";
				names[(System.Int32) IPPorts_Fields.IBM_DB2] = "ibm_db2";
				names[(System.Int32) IPPorts_Fields.NCP] = "ncp";
				names[(System.Int32) IPPorts_Fields.TIMED] = "timed";
				names[(System.Int32) IPPorts_Fields.TEMPO] = "tempo";
				names[(System.Int32) IPPorts_Fields.STX] = "stx";
				names[(System.Int32) IPPorts_Fields.CUSTIX] = "custix";
				names[(System.Int32) IPPorts_Fields.IRC_SERV] = "irc_serv";
				names[(System.Int32) IPPorts_Fields.COURIER] = "courier";
				names[(System.Int32) IPPorts_Fields.CONFERENCE] = "conference";
				names[(System.Int32) IPPorts_Fields.NETNEWS] = "netnews";
				names[(System.Int32) IPPorts_Fields.NETWALL] = "netwall";
				names[(System.Int32) IPPorts_Fields.MM_ADMIN] = "mm_admin";
				names[(System.Int32) IPPorts_Fields.IIOP] = "iiop";
				names[(System.Int32) IPPorts_Fields.OPALIS_RDV] = "opalis_rdv";
				names[(System.Int32) IPPorts_Fields.NMSP] = "nmsp";
				names[(System.Int32) IPPorts_Fields.GDOMAP] = "gdomap";
				names[(System.Int32) IPPorts_Fields.APERTUS_LDP] = "apertus_ldp";
				names[(System.Int32) IPPorts_Fields.UUCP] = "uucp";
				names[(System.Int32) IPPorts_Fields.UUCP_RLOGIN] = "uucp_rlogin";
				names[(System.Int32) IPPorts_Fields.COMMERCE] = "commerce";
				names[(System.Int32) IPPorts_Fields.KLOGIN] = "klogin";
				names[(System.Int32) IPPorts_Fields.KSHELL] = "kshell";
				names[(System.Int32) IPPorts_Fields.APPLEQTCSRVR] = "appleqtcsrvr";
				names[(System.Int32) IPPorts_Fields.DHCPV6_CLIENT] = "dhcpv6_client";
				names[(System.Int32) IPPorts_Fields.DHCPV6_SERVER] = "dhcpv6_server";
				names[(System.Int32) IPPorts_Fields.AFPOVERTCP] = "afpovertcp";
				names[(System.Int32) IPPorts_Fields.IDFP] = "idfp";
				names[(System.Int32) IPPorts_Fields.NEW_RWHO] = "new_rwho";
				names[(System.Int32) IPPorts_Fields.CYBERCASH] = "cybercash";
				names[(System.Int32) IPPorts_Fields.DEVICESHARE] = "deviceshare";
				names[(System.Int32) IPPorts_Fields.PIRP] = "pirp";
				names[(System.Int32) IPPorts_Fields.RTSP] = "rtsp";
				names[(System.Int32) IPPorts_Fields.DSF] = "dsf";
				names[(System.Int32) IPPorts_Fields.REMOTEFS] = "remotefs";
				names[(System.Int32) IPPorts_Fields.OPENVMS_SYSIPC] = "openvms_sysipc";
				names[(System.Int32) IPPorts_Fields.SDNSKMP] = "sdnskmp";
				names[(System.Int32) IPPorts_Fields.TEEDTAP] = "teedtap";
				names[(System.Int32) IPPorts_Fields.RMONITOR] = "rmonitor";
				names[(System.Int32) IPPorts_Fields.MONITOR] = "monitor";
				names[(System.Int32) IPPorts_Fields.CHSHELL] = "chshell";
				names[(System.Int32) IPPorts_Fields.NNTPS] = "nntps";
				names[(System.Int32) IPPorts_Fields.N9PFS] = "9pfs";
				names[(System.Int32) IPPorts_Fields.WHOAMI] = "whoami";
				names[(System.Int32) IPPorts_Fields.STREETTALK] = "streettalk";
				names[(System.Int32) IPPorts_Fields.BANYAN_RPC] = "banyan_rpc";
				names[(System.Int32) IPPorts_Fields.MS_SHUTTLE] = "ms_shuttle";
				names[(System.Int32) IPPorts_Fields.MS_ROME] = "ms_rome";
				names[(System.Int32) IPPorts_Fields.METER] = "meter";
				names[(System.Int32) IPPorts_Fields.METER2] = "meter2";
				names[(System.Int32) IPPorts_Fields.SONAR] = "sonar";
				names[(System.Int32) IPPorts_Fields.BANYAN_VIP] = "banyan_vip";
				names[(System.Int32) IPPorts_Fields.FTP_AGENT] = "ftp_agent";
				names[(System.Int32) IPPorts_Fields.VEMMI] = "vemmi";
				names[(System.Int32) IPPorts_Fields.IPCD] = "ipcd";
				names[(System.Int32) IPPorts_Fields.VNAS] = "vnas";
				names[(System.Int32) IPPorts_Fields.IPDD] = "ipdd";
				names[(System.Int32) IPPorts_Fields.DECBSRV] = "decbsrv";
				names[(System.Int32) IPPorts_Fields.SNTP_HEARTBEAT] = "sntp_heartbeat";
				names[(System.Int32) IPPorts_Fields.BDP] = "bdp";
				names[(System.Int32) IPPorts_Fields.SCC_SECURITY] = "scc_security";
				names[(System.Int32) IPPorts_Fields.PHILIPS_VC] = "philips_vc";
				names[(System.Int32) IPPorts_Fields.KEYSERVER] = "keyserver";
				names[(System.Int32) IPPorts_Fields.IMAP4_SSL] = "imap4_ssl";
				names[(System.Int32) IPPorts_Fields.PASSWORD_CHG] = "password_chg";
				names[(System.Int32) IPPorts_Fields.SUBMISSION] = "submission";
				names[(System.Int32) IPPorts_Fields.CAL] = "cal";
				names[(System.Int32) IPPorts_Fields.EYELINK] = "eyelink";
				names[(System.Int32) IPPorts_Fields.TNS_CML] = "tns_cml";
				names[(System.Int32) IPPorts_Fields.HTTP_ALT] = "http_alt";
				names[(System.Int32) IPPorts_Fields.EUDORA_SET] = "eudora_set";
				names[(System.Int32) IPPorts_Fields.HTTP_RPC_EPMAP] = "http_rpc_epmap";
				names[(System.Int32) IPPorts_Fields.TPIP] = "tpip";
				names[(System.Int32) IPPorts_Fields.CAB_PROTOCOL] = "cab_protocol";
				names[(System.Int32) IPPorts_Fields.SMSD] = "smsd";
				names[(System.Int32) IPPorts_Fields.PTCNAMESERVICE] = "ptcnameservice";
				names[(System.Int32) IPPorts_Fields.SCO_WEBSRVRMG3] = "sco_websrvrmg3";
				names[(System.Int32) IPPorts_Fields.ACP] = "acp";
				names[(System.Int32) IPPorts_Fields.IPCSERVER] = "ipcserver";
				names[(System.Int32) IPPorts_Fields.URM] = "urm";
				names[(System.Int32) IPPorts_Fields.NQS] = "nqs";
				names[(System.Int32) IPPorts_Fields.SIFT_UFT] = "sift_uft";
				names[(System.Int32) IPPorts_Fields.NPMP_TRAP] = "npmp_trap";
				names[(System.Int32) IPPorts_Fields.NPMP_LOCAL] = "npmp_local";
				names[(System.Int32) IPPorts_Fields.NPMP_GUI] = "npmp_gui";
				names[(System.Int32) IPPorts_Fields.HMMP_IND] = "hmmp_ind";
				names[(System.Int32) IPPorts_Fields.HMMP_OP] = "hmmp_op";
				names[(System.Int32) IPPorts_Fields.SSHELL] = "sshell";
				names[(System.Int32) IPPorts_Fields.SCO_INETMGR] = "sco_inetmgr";
				names[(System.Int32) IPPorts_Fields.SCO_SYSMGR] = "sco_sysmgr";
				names[(System.Int32) IPPorts_Fields.SCO_DTMGR] = "sco_dtmgr";
				names[(System.Int32) IPPorts_Fields.DEI_ICDA] = "dei_icda";
				names[(System.Int32) IPPorts_Fields.DIGITAL_EVM] = "digital_evm";
				names[(System.Int32) IPPorts_Fields.SCO_WEBSRVRMGR] = "sco_websrvrmgr";
				names[(System.Int32) IPPorts_Fields.ESCP_IP] = "escp_ip";
				names[(System.Int32) IPPorts_Fields.COLLABORATOR] = "collaborator";
				names[(System.Int32) IPPorts_Fields.AUX_BUS_SHUNT] = "aux_bus_shunt";
				names[(System.Int32) IPPorts_Fields.CRYPTOADMIN] = "cryptoadmin";
				names[(System.Int32) IPPorts_Fields.DEC_DLM] = "dec_dlm";
				names[(System.Int32) IPPorts_Fields.ASIA] = "asia";
				names[(System.Int32) IPPorts_Fields.PASSGO_TIVOLI] = "passgo_tivoli";
				names[(System.Int32) IPPorts_Fields.QMQP] = "qmqp";
				names[(System.Int32) IPPorts_Fields.AMP3_3COM] = "3com_amp3";
				names[(System.Int32) IPPorts_Fields.RDA] = "rda";
				names[(System.Int32) IPPorts_Fields.IPP] = "ipp";
				names[(System.Int32) IPPorts_Fields.BMPP] = "bmpp";
				names[(System.Int32) IPPorts_Fields.SERVSTAT] = "servstat";
				names[(System.Int32) IPPorts_Fields.GINAD] = "ginad";
				names[(System.Int32) IPPorts_Fields.RLZDBASE] = "rlzdbase";
				names[(System.Int32) IPPorts_Fields.LDAPS] = "ldaps";
				names[(System.Int32) IPPorts_Fields.LANSERVER] = "lanserver";
				names[(System.Int32) IPPorts_Fields.MCNS_SEC] = "mcns_sec";
				names[(System.Int32) IPPorts_Fields.MSDP] = "msdp";
				names[(System.Int32) IPPorts_Fields.ENTRUST_SPS] = "entrust_sps";
				names[(System.Int32) IPPorts_Fields.REPCMD] = "repcmd";
				names[(System.Int32) IPPorts_Fields.ESRO_EMSDP] = "esro_emsdp";
				names[(System.Int32) IPPorts_Fields.SANITY] = "sanity";
				names[(System.Int32) IPPorts_Fields.DWR] = "dwr";
				names[(System.Int32) IPPorts_Fields.PSSC] = "pssc";
				names[(System.Int32) IPPorts_Fields.LDP] = "ldp";
				names[(System.Int32) IPPorts_Fields.DHCP_FAILOVER] = "dhcp_failover";
				names[(System.Int32) IPPorts_Fields.RRP] = "rrp";
				names[(System.Int32) IPPorts_Fields.AMINET] = "aminet";
				names[(System.Int32) IPPorts_Fields.OBEX] = "obex";
				names[(System.Int32) IPPorts_Fields.IEEE_MMS] = "ieee_mms";
				names[(System.Int32) IPPorts_Fields.HELLO_PORT] = "hello_port";
				names[(System.Int32) IPPorts_Fields.REPSCMD] = "repscmd";
				names[(System.Int32) IPPorts_Fields.AODV] = "aodv";
				names[(System.Int32) IPPorts_Fields.TINC] = "tinc";
				names[(System.Int32) IPPorts_Fields.SPMP] = "spmp";
				names[(System.Int32) IPPorts_Fields.RMC] = "rmc";
				names[(System.Int32) IPPorts_Fields.TENFOLD] = "tenfold";
				names[(System.Int32) IPPorts_Fields.URL_RENDEZVOUS] = "url_rendezvous";
				names[(System.Int32) IPPorts_Fields.MAC_SRVR_ADMIN] = "mac_srvr_admin";
				names[(System.Int32) IPPorts_Fields.HAP] = "hap";
				names[(System.Int32) IPPorts_Fields.PFTP] = "pftp";
				names[(System.Int32) IPPorts_Fields.PURENOISE] = "purenoise";
				names[(System.Int32) IPPorts_Fields.SECURE_AUX_BUS] = "secure_aux_bus";
				names[(System.Int32) IPPorts_Fields.SUN_DR] = "sun_dr";
				names[(System.Int32) IPPorts_Fields.MDQS] = "mdqs";
				names[(System.Int32) IPPorts_Fields.DOOM] = "doom";
				names[(System.Int32) IPPorts_Fields.DISCLOSE] = "disclose";
				names[(System.Int32) IPPorts_Fields.MECOMM] = "mecomm";
				names[(System.Int32) IPPorts_Fields.MEREGISTER] = "meregister";
				names[(System.Int32) IPPorts_Fields.VACDSM_SWS] = "vacdsm_sws";
				names[(System.Int32) IPPorts_Fields.VACDSM_APP] = "vacdsm_app";
				names[(System.Int32) IPPorts_Fields.VPPS_QUA] = "vpps_qua";
				names[(System.Int32) IPPorts_Fields.CIMPLEX] = "cimplex";
				names[(System.Int32) IPPorts_Fields.ACAP] = "acap";
				names[(System.Int32) IPPorts_Fields.DCTP] = "dctp";
				names[(System.Int32) IPPorts_Fields.VPPS_VIA] = "vpps_via";
				names[(System.Int32) IPPorts_Fields.VPP] = "vpp";
				names[(System.Int32) IPPorts_Fields.GGF_NCP] = "ggf_ncp";
				names[(System.Int32) IPPorts_Fields.MRM] = "mrm";
				names[(System.Int32) IPPorts_Fields.ENTRUST_AAAS] = "entrust_aaas";
				names[(System.Int32) IPPorts_Fields.ENTRUST_AAMS] = "entrust_aams";
				names[(System.Int32) IPPorts_Fields.XFR] = "xfr";
				names[(System.Int32) IPPorts_Fields.CORBA_IIOP] = "corba_iiop";
				names[(System.Int32) IPPorts_Fields.CORBA_IIOP_SSL] = "corba_iiop_ssl";
				names[(System.Int32) IPPorts_Fields.MDC_PORTMAPPER] = "mdc_portmapper";
				names[(System.Int32) IPPorts_Fields.HCP_WISMAR] = "hcp_wismar";
				names[(System.Int32) IPPorts_Fields.ASIPREGISTRY] = "asipregistry";
				names[(System.Int32) IPPorts_Fields.REALM_RUSD] = "realm_rusd";
				names[(System.Int32) IPPorts_Fields.NMAP] = "nmap";
				names[(System.Int32) IPPorts_Fields.VATP] = "vatp";
				names[(System.Int32) IPPorts_Fields.MSEXCH_ROUTING] = "msexch_routing";
				names[(System.Int32) IPPorts_Fields.HYPERWAVE_ISP] = "hyperwave_isp";
				names[(System.Int32) IPPorts_Fields.CONNENDP] = "connendp";
				names[(System.Int32) IPPorts_Fields.HA_CLUSTER] = "ha_cluster";
				names[(System.Int32) IPPorts_Fields.IEEE_MMS_SSL] = "ieee_mms_ssl";
				names[(System.Int32) IPPorts_Fields.RUSHD] = "rushd";
				names[(System.Int32) IPPorts_Fields.UUIDGEN] = "uuidgen";
				names[(System.Int32) IPPorts_Fields.OLSR] = "olsr";
				names[(System.Int32) IPPorts_Fields.ACCESSNETWORK] = "accessnetwork";
				names[(System.Int32) IPPorts_Fields.ELCSD] = "elcsd";
				names[(System.Int32) IPPorts_Fields.AGENTX] = "agentx";
				names[(System.Int32) IPPorts_Fields.SILC] = "silc";
				names[(System.Int32) IPPorts_Fields.BORLAND_DSJ] = "borland_dsj";
				names[(System.Int32) IPPorts_Fields.ENTRUST_KMSH] = "entrust_kmsh";
				names[(System.Int32) IPPorts_Fields.ENTRUST_ASH] = "entrust_ash";
				names[(System.Int32) IPPorts_Fields.CISCO_TDP] = "cisco_tdp";
				names[(System.Int32) IPPorts_Fields.NETVIEWDM1] = "netviewdm1";
				names[(System.Int32) IPPorts_Fields.NETVIEWDM2] = "netviewdm2";
				names[(System.Int32) IPPorts_Fields.NETVIEWDM3] = "netviewdm3";
				names[(System.Int32) IPPorts_Fields.NETGW] = "netgw";
				names[(System.Int32) IPPorts_Fields.NETRCS] = "netrcs";
				names[(System.Int32) IPPorts_Fields.FLEXLM] = "flexlm";
				names[(System.Int32) IPPorts_Fields.FUJITSU_DEV] = "fujitsu_dev";
				names[(System.Int32) IPPorts_Fields.RIS_CM] = "ris_cm";
				names[(System.Int32) IPPorts_Fields.KERBEROS_ADM] = "kerberos_adm";
				names[(System.Int32) IPPorts_Fields.RFILE] = "rfile";
				names[(System.Int32) IPPorts_Fields.PUMP] = "pump";
				names[(System.Int32) IPPorts_Fields.QRH] = "qrh";
				names[(System.Int32) IPPorts_Fields.RRH] = "rrh";
				names[(System.Int32) IPPorts_Fields.TELL] = "tell";
				names[(System.Int32) IPPorts_Fields.NLOGIN] = "nlogin";
				names[(System.Int32) IPPorts_Fields.CON] = "con";
				names[(System.Int32) IPPorts_Fields.NS] = "ns";
				names[(System.Int32) IPPorts_Fields.RXE] = "rxe";
				names[(System.Int32) IPPorts_Fields.QUOTAD] = "quotad";
				names[(System.Int32) IPPorts_Fields.CYCLESERV] = "cycleserv";
				names[(System.Int32) IPPorts_Fields.OMSERV] = "omserv";
				names[(System.Int32) IPPorts_Fields.WEBSTER] = "webster";
				names[(System.Int32) IPPorts_Fields.PHONEBOOK] = "phonebook";
				names[(System.Int32) IPPorts_Fields.VID] = "vid";
				names[(System.Int32) IPPorts_Fields.CADLOCK] = "cadlock";
				names[(System.Int32) IPPorts_Fields.RTIP] = "rtip";
				names[(System.Int32) IPPorts_Fields.CYCLESERV2] = "cycleserv2";
				names[(System.Int32) IPPorts_Fields.SUBMIT] = "submit";
				names[(System.Int32) IPPorts_Fields.RPASSWD] = "rpasswd";
				names[(System.Int32) IPPorts_Fields.ENTOMB] = "entomb";
				names[(System.Int32) IPPorts_Fields.WPAGES] = "wpages";
				names[(System.Int32) IPPorts_Fields.WPGS] = "wpgs";
				names[(System.Int32) IPPorts_Fields.QSC] = "qsc";
				names[(System.Int32) IPPorts_Fields.MDBS_DAEMON] = "mdbs_daemon";
				names[(System.Int32) IPPorts_Fields.ITM_MCELL_S] = "itm_mcell_s";
				names[(System.Int32) IPPorts_Fields.PKIX_3_CA_RA] = "pkix_3_ca_ra";
				names[(System.Int32) IPPorts_Fields.DHCP_FAILOVER2] = "dhcp_failover2";
				names[(System.Int32) IPPorts_Fields.RSYNC] = "rsync";
				names[(System.Int32) IPPorts_Fields.ICLCNET_LOCATE] = "iclcnet_locate";
				names[(System.Int32) IPPorts_Fields.ICLCNET_SVINFO] = "iclcnet_svinfo";
				names[(System.Int32) IPPorts_Fields.ACCESSBUILDER] = "accessbuilder";
				names[(System.Int32) IPPorts_Fields.CDDBP] = "cddbp";
				names[(System.Int32) IPPorts_Fields.OMGINITIALREFS] = "omginitialrefs";
				names[(System.Int32) IPPorts_Fields.SMPNAMERES] = "smpnameres";
				names[(System.Int32) IPPorts_Fields.IDEAFARM_CHAT] = "ideafarm_chat";
				names[(System.Int32) IPPorts_Fields.IDEAFARM_CATCH] = "ideafarm_catch";
				names[(System.Int32) IPPorts_Fields.XACT_BACKUP] = "xact_backup";
				names[(System.Int32) IPPorts_Fields.FTPS_DATA] = "ftps_data";
				names[(System.Int32) IPPorts_Fields.FTPS] = "ftps";
				names[(System.Int32) IPPorts_Fields.NAS] = "nas";
				names[(System.Int32) IPPorts_Fields.TELNETS] = "telnets";
				names[(System.Int32) IPPorts_Fields.IMAPS] = "imaps";
				names[(System.Int32) IPPorts_Fields.IRCS] = "ircs";
				names[(System.Int32) IPPorts_Fields.POP3S] = "pop3s";
				names[(System.Int32) IPPorts_Fields.VSINET] = "vsinet";
				names[(System.Int32) IPPorts_Fields.MAITRD] = "maitrd";
				names[(System.Int32) IPPorts_Fields.BUSBOY] = "busboy";
				names[(System.Int32) IPPorts_Fields.GARCON] = "garcon";
				names[(System.Int32) IPPorts_Fields.PUPROUTER] = "puprouter";
				names[(System.Int32) IPPorts_Fields.CADLOCK2] = "cadlock2";
				names[(System.Int32) IPPorts_Fields.SURF] = "surf";
				names[(System.Int32) IPPorts_Fields.RESERVED_1023] = "1023r";
				names[(System.Int32) IPPorts_Fields.RESERVED_1024] = "1024r";
				names[(System.Int32) IPPorts_Fields.SOCKS] = "socks";
				names[(System.Int32) IPPorts_Fields.HSRP] = "hsrp";
				names[(System.Int32) IPPorts_Fields.CVS] = "cvs";
				names[(System.Int32) IPPorts_Fields.AISES] = "aises";
				names[(System.Int32) IPPorts_Fields.X11] = "X11";
			}
		}
	}
}