﻿using System;
using System.Collections.Generic;
using SysConfig = System.Configuration;
using System.Linq;

namespace Terminals.Configuration
{
    internal static partial class Settings
    {
        /// <summary>
        /// Gets alphabeticaly sorted array of tags resolved from Tags store.
        /// Since version 2. only for updates. Use new persistence instead.
        /// </summary>
        internal static string[] Tags
        {
            get
            {
                List<string> tags = GetSection().Tags.ToList();
                tags.Sort();
                return tags.ToArray();
            }
        }

        private static void AddTagsToSettings(List<string> tags)
        {
            foreach (String tag in tags)
            {
                if (String.IsNullOrEmpty(tag))
                    continue;

                AddTagToSettings(tag);
            }
        }

        /// <summary>
        /// Adds tag to the tags collection if it already isnt there.
        /// If the tag is in database, than it returns empty string, otherwise the commited tag.
        /// </summary>
        private static void AddTagToSettings(String tag)
        {
            if (AutoCaseTags)
                tag = ToTitleCase(tag);
            if (Tags.Contains(tag))
                return;

            GetSection().Tags.AddByName(tag);
            SaveImmediatelyIfRequested();
        }

        private static void DeleteTags()
        {
            List<string> tagsToDelete = Tags.ToList();
            DeleteTagsFromSettings(tagsToDelete);
        }

        private static List<string> DeleteTagsFromSettings(List<string> tagsToDelete)
        {
            List<string> deletedTags = new List<string>();
            foreach (String tagTodelete in tagsToDelete)
            {
                if (GetNumberOfFavoritesUsingTag(tagTodelete) > 0)
                    continue;

                deletedTags.Add(DeleteTagFromSettings(tagTodelete));
            }
            return deletedTags;
        }

        /// <summary>
        /// Removes the tag from settings, but doesnt send the Tag removed event
        /// </summary>
        private static String DeleteTagFromSettings(String tagToDelete)
        {
            if (AutoCaseTags)
                tagToDelete = ToTitleCase(tagToDelete);
            GetSection().Tags.DeleteByName(tagToDelete);
            SaveImmediatelyIfRequested();
            return tagToDelete;
        }

        private static Int32 GetNumberOfFavoritesUsingTag(String tagToRemove)
        {
            var favorites = GetFavorites().ToList();
            return favorites.Where(favorite => favorite.TagList.Contains(tagToRemove))
                .Count();
        }
    }
}
